// ClientThread.cpp : implementation file
//

#include "stdafx.h"
#include "MessengerServer.h"
#include "ClientThread.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CClientThread

IMPLEMENT_DYNCREATE(CClientThread, CWinThread)

CClientThread::CClientThread()
{
}

CClientThread::~CClientThread()
{
}

BOOL CClientThread::InitInstance()
{
	// TODO:  perform and per-thread initialization here
	return TRUE;
}

int CClientThread::ExitInstance()
{
	return CWinThread::ExitInstance();
}

BEGIN_MESSAGE_MAP(CClientThread, CWinThread)
	//{{AFX_MSG_MAP(CClientThread)
		// NOTE - the ClassWizard will add and remove mapping macros here.
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CClientThread message handlers

int CClientThread::Run() 
{
	return CWinThread::Run();
}



/**************************************************************************/
// Method Name	: ReceiveMessagePacket
// Description	: Method for receiving 1 message packet sent by the client
// Inputs		: Client socket from where data is coming
// Output		: Receives a message packet from client and sends it 
//					for processing
// Returns		: void
/**************************************************************************/
bool CClientThread::ReceiveMessagePacket(CMessengerClientSocket *pCurrentClient)
{
	CMessagePacket msg;

	// If the To party is found in the connections list, then go ahead
	if(pCurrentClient)
	{
		TRY
		{
			msg.Serialize(*pCurrentClient->m_pArIn);
			Sleep(100);
		}
		CATCH(CFileException, e)
		{
			return FALSE;
		}
		END_CATCH

		ProcessMessagePacket(msg, pCurrentClient);

		return TRUE;
	}
	return FALSE;
}



/**************************************************************************/
// Method Name	: ProcessMessagePacket
// Description	: Method for processing a message packet sent by the client
// Inputs		: Message packet to be processed
//				  Client socket from where data is coming
// Output		: Prints the message log into the log window of server
//				  Reads the reason flag in the incoming message packet
//				  Depending on this flag, does various actions as follows:
//					Flag Meaning			Action
//					-----------------------------------------------------
//					L	 Login request		Process login request
//					R	 Registration		Process registration request
//						 request			
//					M	 A client sent		Send message ahead
//						 msg to another		
//					SL	 Friends list		Send friend's list
//						 requested
//					SLO	 Offline messages	Send all offline messages
//						 requested
//					ON	 Send next offline	Send next offline msg
//						 message
//					X	 Logoff				Logs of client
//					AF	 Add friend req		Send the request for
//											processing to target
//					AFA	 Accept and Add		Add both To and From
//											parties as each other's
//											friends
//					AFJ	 Just Add			Add From party as To 
//											party's friend
//					AFD	 Don't add			Send msg to To party
//											that req is declined
//					C	 Change password	Change From party's password
//					DF	 Delete Friends		Delete friends mentioned in
//											msg packet
//					BU	 Block user			Add the To name into blocked 
//											users list in memory
//					BUP	 Block user			Add the To name into blocked 
//						 permanently		users list in memory AND in 
//											database
//					BUL	 Blocked Users		Send blocked users list to
//						 list requested		client
//					BUR	 Blocked Users		Remove selected blocked users
//						 removal						
// Returns		: true	- success
//				  false - otherwise
/**************************************************************************/
bool CClientThread::ProcessMessagePacket(CMessagePacket& msg, CMessengerClientSocket *pCurrentClient)
{
	// Log this message
	LogMessage(msg);

	// Get doc pointer
	CMessengerServerView *view = GetViewPtr();
	CMessengerServerDoc *pDoc = view->GetDocument();

	if(msg.m_strReasonFlag == "L")		// Someone is trying to Login
		ProcessLoginRequest(msg, pCurrentClient, pDoc);

	else if(msg.m_strReasonFlag == "R")		// Someone is trying to Register
		ProcessRegistrationRequest(msg, pCurrentClient, pDoc);

	else if(msg.m_strReasonFlag == "M")		// Someone is trying to send a message
	{
		ProcessMessageFromOneUserToAnother(msg, pCurrentClient, pDoc);
	}
	
	else if(msg.m_strReasonFlag == "SL")	// Someone is Logged on and is requesting a friends list
		SendListOfFriendsToThisLogin(msg, pDoc);	

	else if(msg.m_strReasonFlag == "SLO")	// Someone has received a friends list and is requesting offline messages
		// Copy all the offline messages for this user from the main list in doc object
		GetAllOfflineMessagesForThisUser(msg, pDoc, true);

	else if(msg.m_strReasonFlag == "ON")	
		// Send next offline messages for this user to him
		GetAllOfflineMessagesForThisUser(msg, pDoc, false);

	else if(msg.m_strReasonFlag == "X")		// Someone has logged out
		ProcessLogoffRequest(msg, pCurrentClient, pDoc);

	else if(msg.m_strReasonFlag == "AF")		// Someone has added a friend
		AskClientForAddPermission(msg, pDoc);
	
	else if(msg.m_strReasonFlag == "AFA")		// Added friend has given permission to be added and also
												// added requester as his friend
		ProcessAddAFriendRequest(msg, true, pCurrentClient, pDoc);
	

	else if(msg.m_strReasonFlag == "AFJ")		// Added friend has given permission to be added but doesn't
												// want requester to be added as his friend
		ProcessAddAFriendRequest(msg, false, pCurrentClient, pDoc);
	
	else if(msg.m_strReasonFlag == "AFD")		// Added friend has declined request to be added 
	{
		CString str = "User " + msg.m_strFrom + " has declined your add friend request";
		SendStatusMessageToClient(str, "FA", msg.m_strTo, pCurrentClient, pDoc);
	}

	else if(msg.m_strReasonFlag == "C")			// Change password request
		ProcessChangePasswordRequest(msg, pCurrentClient, pDoc);

	else if(msg.m_strReasonFlag == "DF")		// Delete some friends
		ProcessDeleteFriendsRequest(msg, pCurrentClient, pDoc);
	else if(msg.m_strReasonFlag == "BUP")		// Block sender permanently
		ProcessBlockUserRequest(msg, pCurrentClient, pDoc, true);
	else if(msg.m_strReasonFlag == "BU")		// Block sender for this session only
		ProcessBlockUserRequest(msg, pCurrentClient, pDoc, false);
	else if(msg.m_strReasonFlag == "BUL")		// Blocked users list requested by client
		SendBlockUserList(msg, pCurrentClient, pDoc);
	else if(msg.m_strReasonFlag == "BUR")		// Remove some Blocked users from list
		RemoveSelBlockedUsers(msg, pCurrentClient, pDoc);

	return TRUE;
}



/**************************************************************************/
// Method Name	: GetViewPtr
// Description	: Method for finding view pointer
// Inputs		: None
// Output		: None
// Returns		: view pointer
/**************************************************************************/
CMessengerServerView *CClientThread::GetViewPtr()
{
	CMainFrame *wnd = (CMainFrame *)::AfxGetMainWnd();

	CMessengerServerView *pView = (CMessengerServerView *)wnd->GetActiveView();

	return pView;
}



/**************************************************************************/
// Method Name	: ProcessLoginRequest
// Description	: Method for processing Login Request
// Inputs		: Message packet sent by the user
//				  Client socket where the msg was received
// Output		: Loads user dll
//				  Validates username and password from database using this dll
//				  If login is valid, 
//					checks whether it's active
//					If not active (user not logged in), logs him in
//					Makes the entry into logged in users' list
//					Removes user's entry from not logged in user's list
//					Sends login success message to user
//					Notifies all users abt this user logging in
//				  If login invalid or user already logged in
//					Sends failure message over the socket.
// Returns		: void
/**************************************************************************/
void CClientThread::ProcessLoginRequest(CMessagePacket& msg, CMessengerClientSocket *pCurrentClient, CMessengerServerDoc *m_pDoc)
{
	// Load the User dll
	LoadLibrary("User.dll");

	// Validate UserName, password
	BOOL res = isValid(msg.m_strFrom, msg.m_strTo);

	if(res)
	{
		// Check if this user is already logged in
		msg.m_strTo = msg.m_strFrom;
		CMessengerClientSocket *sock = FindOutTheClient(msg, m_pDoc);
		if(sock)
		{
			SendStatusMessageToClient("Someone is already logged in with that name", "FL", "", pCurrentClient, m_pDoc);
			return;
		}

		// Keep the Login details in the client socket
		pCurrentClient->m_strUserName = msg.m_strFrom;

		// Set the status as 'logged in' in the client socket
		pCurrentClient->m_strLoggedStatus = "L";

		// Add the new socket to the client's list
		m_pDoc->m_ClientsList.AddTail(pCurrentClient);

		// Remove this socket from sockets list
		RemoveFromSocketsList(pCurrentClient, m_pDoc);

		// Send acknowledgement to client
		SendStatusMessageToClient("Login Success", "SL", msg.m_strFrom, pCurrentClient, m_pDoc);

		// Broadcast this news to all friends
		TellFriendsAbtLogin(msg, m_pDoc);

		// Put this user's name into server's online users list
		AddUserToList(msg, m_pDoc, ONLINE_USERS_LIST, false);
	}
	else
		SendStatusMessageToClient("Login Failed", "FL", "", pCurrentClient, m_pDoc);
}



/**************************************************************************/
// Method Name	: SendStatusMessageToClient
// Description	: Method to send acknowledgement to the client
// Inputs		: Message string to be sent to the client
//				  Reason flag.
//				  Client's user name
//				  Client socket where the msg is to be sent
//				  Doc pointer
// Output		: Sends message over the socket.
// Returns		: TRUE	- Success
//				  FALSE - Otherwise
/**************************************************************************/
bool CClientThread::SendStatusMessageToClient(CString msg, CString status, CString strRecipient, CMessengerClientSocket *sock, CMessengerServerDoc *m_pDoc)
{
	CMessagePacket msgPacket;

	// Prepare message
	msgPacket.m_strFrom = msg;
	msgPacket.m_strReasonFlag = status;	
	msgPacket.m_strTo = strRecipient;

	if(strRecipient != "")
	{
		// Find the client's socket
		CMessengerClientSocket *clientSockPtr = FindOutTheClient(msgPacket, m_pDoc);
		
		// Send message to the client
		if(clientSockPtr)
			return SendMessagePacket(msgPacket, clientSockPtr, m_pDoc);
	}

	return SendMessagePacket(msgPacket, sock, m_pDoc);
}



/**************************************************************************/
// Method Name	: SendMessagePacket
// Description	: Method to send 1 message packet to the client
// Inputs		: Message packet to be sent to the client
//				  Client socket where the msg is to be sent
//				  Doc pointer
// Output		: Sends message over the socket.
// Returns		: TRUE	- Success
//				  FALSE - Otherwise
// Note			: Do not remove the TRACE call. It is there to induce
//				  some delay after sending msg. If it's not there, msg
//				  transmission will be unreliable.
/**************************************************************************/
bool CClientThread::SendMessagePacket(CMessagePacket& msgPacket, CMessengerClientSocket *clientSocket, CMessengerServerDoc *m_pDoc)
{
	TRY
	{
		msgPacket.Serialize(*clientSocket->m_pArOut);
		clientSocket->m_pArOut->Flush();

		Sleep(100);
		//TRACE("Message Sent");
	}
	CATCH(CFileException, e)
	{
		return FALSE;
	}
	END_CATCH

	return TRUE;
}



/**************************************************************************/
// Method Name	: FindOutTheClient
// Description	: Method to find out the socket for a client
// Inputs		: Message packet containing name of the client
//				  Doc pointer
// Output		: Finds out the clients socket address from the logged
//				  in client's list
// Returns		: Client socket corresponding to the client name if found
//				  NULL otherwise
/**************************************************************************/
CMessengerClientSocket *CClientThread::FindOutTheClient(CMessagePacket& msg, CMessengerServerDoc *m_pDoc)
{	
	if(m_pDoc->m_ClientsList.IsEmpty())
		return NULL;

	// See if the To party is online. For this iterate thru the client
	// sockets list
	CMessengerClientSocket *sockPtr = (CMessengerClientSocket *)m_pDoc->m_ClientsList.GetHead();
	CMessengerClientSocket *tailPtr = (CMessengerClientSocket *)m_pDoc->m_ClientsList.GetTail();

	// Store the head position
	POSITION pos = m_pDoc->m_ClientsList.GetHeadPosition();

	while(1)
	{
		// If current position is the To party and he is logged in
		if((sockPtr->m_strUserName == msg.m_strTo) && (sockPtr->m_strLoggedStatus == "L"))
			return sockPtr;

		// If we ve reached end of list
		if(sockPtr == tailPtr)
			break;

		// Else go ahead 1 position
		sockPtr = (CMessengerClientSocket *)m_pDoc->m_ClientsList.GetNext(pos);
	}

	// To party not found
	return NULL;
}



/**************************************************************************/
// Method Name	: RemoveFromSocketsList
// Description	: Method to remove 1 socket from NOT logged
//				  in client's list
// Inputs		: Pointer to the socket which is to be removed
//				  Doc pointer
// Output		: Finds out the clients socket's position in the NOT logged
//				  in client's list
//				  Removes that socket it found
// Returns		: Removed Client socket if success
//				  NULL otherwise
// Note			: Insted of using Remove call of the CListPtr, we call our own
//				  method RemoveAListMember to remove socket list member. This
//				  is to make this operation THREAD SAFE.
/**************************************************************************/
CMessengerClientSocket *CClientThread::RemoveFromSocketsList(CMessengerClientSocket *socket, CMessengerServerDoc *m_pDoc)
{
	if(m_pDoc->m_SocketsList.IsEmpty())
		return NULL;

	CMessengerClientSocket *sockPtr = (CMessengerClientSocket *)m_pDoc->m_SocketsList.GetHead();
	CMessengerClientSocket *tailPtr = (CMessengerClientSocket *)m_pDoc->m_SocketsList.GetTail();

	POSITION pos = m_pDoc->m_SocketsList.GetHeadPosition();
	sockPtr = (CMessengerClientSocket *)m_pDoc->m_SocketsList.GetNext(pos);

	while(1)
	{
		if(sockPtr == socket)			// If this socket found
		{
			// Remove from the list and return it's address
			pos = m_pDoc->m_SocketsList.Find(sockPtr);
			RemoveAListMember(&(m_pDoc->m_SocketsList), pos, m_pDoc, HANDLE_TO_SOCKETS_LIST_MUTEX);

			return sockPtr;
		}

		// If we are at the end of list, get out
		if(sockPtr == tailPtr)
			break;

		// Otherwise go ahead 1 position
		sockPtr = (CMessengerClientSocket *)m_pDoc->m_SocketsList.GetNext(pos);
	}

	return NULL;
}



/**************************************************************************/
// Method Name	: TellFriendsAbtLogin
// Description	: Method to broadcast to all connected clients that a user
//				  has logged in. Only his friends will process the message
//				  Others will ignore it.
// Inputs		: Message packet from newly logged on user.
//				  Doc pointer
// Output		: Sends message to all clients abt this user logging in
// Returns		: void
/**************************************************************************/
void CClientThread::TellFriendsAbtLogin(CMessagePacket& msg, CMessengerServerDoc *m_pDoc)
{
	CMessengerClientSocket *sockPtr = (CMessengerClientSocket *)m_pDoc->m_ClientsList.GetHead();
	CMessengerClientSocket *tailPtr = (CMessengerClientSocket *)m_pDoc->m_ClientsList.GetTail();

	// Store the head position
	POSITION pos = m_pDoc->m_ClientsList.GetHeadPosition();
	sockPtr = (CMessengerClientSocket *)m_pDoc->m_ClientsList.GetNext(pos);

	while(1)
	{
		// Do not tell to self
		if(sockPtr->m_strUserName == msg.m_strFrom)
		{
			if(sockPtr == tailPtr)
				break;

			sockPtr = (CMessengerClientSocket *)m_pDoc->m_ClientsList.GetNext(pos);
			continue;
		}
	
		// Tell about this user logging on to the friends
		msg.m_strTo = sockPtr->m_strUserName;
		msg.m_strReasonFlag = "SN";
		SendMessagePacket(msg, sockPtr, m_pDoc);

		// If we ve reached end of list
		if(sockPtr == tailPtr)
			break;

		// Else go ahead 1 position
		sockPtr = (CMessengerClientSocket *)m_pDoc->m_ClientsList.GetNext(pos);
	}
}



/**************************************************************************/
// Method Name	: ProcessRegistrationRequest
// Description	: Method to process Registration Request
// Inputs		: Message packet from newly logged on user.
//				  Socket from where this packet came in
//				  Doc pointer
// Output		: Loads User dll for database ops
//				  Adds the user
//				  Sends success / failure msg to client
// Returns		: void
/**************************************************************************/
void CClientThread::ProcessRegistrationRequest(CMessagePacket& msg, CMessengerClientSocket *pCurrentClient, CMessengerServerDoc *m_pDoc)
{
	// Try to add the user using the User dll
	LoadLibrary("User.dll");
	int ret = addUser(msg.m_strFrom, msg.m_strTo);

	switch(ret)
	{
	case 0:
		SendStatusMessageToClient("User Successfully Added", "SR", "", pCurrentClient, m_pDoc);
		GetViewPtr()->m_lstOffLineUsers.InsertItem(0, msg.m_strFrom);
		break;
	case 1:
		SendStatusMessageToClient("User already exists", "FR", "", pCurrentClient, m_pDoc);
		break;
	case 2:
		SendStatusMessageToClient("Failure adding User", "FR", "", pCurrentClient, m_pDoc);
		break;
	}
}



/**************************************************************************/
// Method Name	: SendMessageToRecepient
// Description	: Method to send a single message packet to a client
// Inputs		: Message packet to be sent to the client.
//				  Doc pointer
// Output		: Finds out the client from Logged in users list
//				  If user not found
//					Adds the message to offline messages
//				  otherwise
//					Send this message to that client's socket
//				  Sends the msg packet to client
// Returns		: void
/**************************************************************************/
void CClientThread::SendMessageToRecepient(CMessagePacket& msg, CMessengerServerDoc *m_pDoc)
{
	CMessengerClientSocket *clientSockPtr = FindOutTheClient(msg, m_pDoc);
	
	// If To party is online, send message to him
	if(clientSockPtr)
	{
		SendMessagePacket(msg, clientSockPtr, m_pDoc);
	}
	// Otherwise keep this message as offline message
	else
	{
		CMessagePacket *pMsg = new CMessagePacket(msg);

		// Keep this message as offline message for the To id
		// Some special treatment for message from another user.
		if((pMsg->m_strReasonFlag.GetLength() > 1) && (pMsg->m_strReasonFlag.GetAt(1) == 'M'))
			pMsg->m_strReasonFlag.Insert(0, "O");			// This is offline message

		m_pDoc->m_OfflineMessagesList.AddTail(pMsg);
	}
}



/**************************************************************************/
// Method Name	: SendListOfFriendsToThisLogin
// Description	: Method to send friend's list to a client
// Inputs		: Message packet to be sent to the client.
//				  Doc pointer
// Output		: Reads list of all friends into the msg packet. The
//					StringList var of msg packet will contain friend's list
//				  Checks all friend's for online status. If friend is online
//					append '*' char to his name, so that client can differentiate
//					between online and offline friends.
//				  Sends the msg packet to client when the whole friend's list is
//					constructed
// Returns		: void
/**************************************************************************/
void CClientThread::SendListOfFriendsToThisLogin(CMessagePacket& msg, CMessengerServerDoc *m_pDoc)
{
	msg.m_strTo = msg.m_strFrom;
	msg.m_strReasonFlag = "SLU";		// Users list

	// First get all the friends list
	CStringList friendsList;

	GetListOfAllFriends(msg, friendsList, m_pDoc);

	CString str;

	while(1)
	{
		// If there's no element in the friend's list
		if(friendsList.IsEmpty())
			break;

		// Get the fisrt item out
		str = friendsList.RemoveHead();

		// Find out if this friend is online
		CMessagePacket msgPacket;
		msgPacket.m_strTo = str;
		CMessengerClientSocket *sockPtr = FindOutTheClient(msgPacket, m_pDoc);

		// If he is online, add a tag to his name
		if(sockPtr)
			str += "*";

		// Add the friend's name to the list in the message
		msg.m_msgList.AddTail(str);
	}

	CMessengerClientSocket *recipientSock = FindOutTheClient(msg, m_pDoc);
	SendMessagePacket(msg, recipientSock, m_pDoc);
}



/**************************************************************************/
// Method Name	: GetListOfAllFriends
// Description	: Method to read friend's list into a msg pcket
// Inputs		: Message packet to be sent to the client.
//				  A StringList var to keep friends' names
//				  Doc pointer
// Output		: Loads User dll for database ops
//				  Reads list of all friends into the msg packet. The
//					StringList var of msg packet will contain friend's list
//					Checks all friend's for online status. If friend is online
//					append '*' char to his name, so that client can differentiate
//					between online and offline friends.
// Returns		: void
/**************************************************************************/
void CClientThread::GetListOfAllFriends(CMessagePacket& msg, CStringList& friendsList, CMessengerServerDoc *m_pDoc)
{
	CUserRecord m_UserRec;

	TRY
	{
		if(m_UserRec.IsOpen())
		{
			m_UserRec.Close();
		}

		// Query to get all the friends of this user
		CString strQuery = "Select UserName, FriendName from Friends where UserName = '" + msg.m_strFrom + "'";

		m_UserRec.Open(AFX_DB_USE_DEFAULT_TYPE, strQuery);

		// Check whether it's an empty recordset
		if(m_UserRec.IsBOF())
			return;

		// Traverse thru the record set
		while(!m_UserRec.IsEOF())
		{
			// If this name is not the same as the recipient, go ahead
			// (We don't want the self name added in the friend's list)
			if(msg.m_strFrom != m_UserRec.m_Password)
			{
				CMessagePacket msgPack;
				msgPack.m_strTo = m_UserRec.m_Password;

				// Check if the user is online
				if(FindOutTheClient(msgPack, m_pDoc))
				{
					// Add the name to friend's list with Online tag
					CString str = m_UserRec.m_Password + "*";
					friendsList.AddTail(str);
				}
				else
				{
					// Add the name to friend's list without online tag
					CString str = m_UserRec.m_Password;
					friendsList.AddTail(str);
				}
			}

			m_UserRec.MoveNext();
		}

		// Close the record set
		m_UserRec.Close();
	}
	CATCH(CDBException, edb)
	{
		char err[100];
		edb->GetErrorMessage(err, 100);
		::AfxMessageBox("DB Exception = ");
		::AfxMessageBox(err);
	}
	CATCH(CMemoryException, emem)
	{
		char err[100];
		emem->GetErrorMessage(err, 100);
		::AfxMessageBox("Memory Exception = ");
		::AfxMessageBox(err);
	}
	END_CATCH
}



/**************************************************************************/
// Method Name	: GetAllOfflineMessagesForThisUser
// Description	: Method to send offline messages to a user
// Inputs		: Message packet coming from the requester.
//				  Doc pointer
//				  A flag which will be used to indicate to client when to stop
// Output		: Reads thru offline messages list and if a msg belongs to the
//					requester, sends it to him
//				  Removes the sent msg from the list. To ensure this op is
//					Thread safe, we call our owan method RemoveAListMember insted
//					of RemoveAt list member of CListPtr class.
// Returns		: void
// Note			: This method maintains a separate list of all offline msgs specific
//					to this user. It's a subset of the whole big list for all user's
//					msgs. This method gets all msgs for this user from the big list
//					and stores into this local list. Therefore this list is made
//					static.
/**************************************************************************/
void CClientThread::GetAllOfflineMessagesForThisUser(CMessagePacket& msg, CMessengerServerDoc *m_pDoc, bool isGettingMessages)
{
	static CPtrList offlineMessagesList;

	// If we have already got all messages and want to just send the next one
	if(!isGettingMessages)
	{
		SendNextOfflineMessageForThisUser(msg, m_pDoc, offlineMessagesList);
		return;
	}

	if(m_pDoc->m_OfflineMessagesList.IsEmpty())
		return;

	CMessagePacket *sockPtr = (CMessagePacket *)m_pDoc->m_OfflineMessagesList.GetHead();
	CMessagePacket *tailPtr = (CMessagePacket *)m_pDoc->m_OfflineMessagesList.GetTail();

	POSITION pos = m_pDoc->m_OfflineMessagesList.Find(sockPtr);

	sockPtr = (CMessagePacket *)m_pDoc->m_OfflineMessagesList.GetNext(pos);
	CMessagePacket *nextPtr = sockPtr;

	// The next while loop will end when this is true
	bool bBreakVar = false;

	while(1)
	{
		// If this message belongs to this user
		// Copy this pointer from Doc's main list to this thread's list
		if(!bBreakVar)
		{
			if(sockPtr->m_strTo == msg.m_strFrom)
			{
				offlineMessagesList.AddTail(sockPtr);

				// Remove this message from the main offline messages list
				RemoveAListMember(&(m_pDoc->m_OfflineMessagesList), m_pDoc->m_OfflineMessagesList.Find(sockPtr), m_pDoc, HANDLE_TO_OFFLINE_MESSAGES_LIST_MUTEX);
			}
		}
		else
			break;

		// Record the next position
		if(nextPtr != tailPtr)
			nextPtr = (CMessagePacket *)m_pDoc->m_OfflineMessagesList.GetNext(pos);
		else
			bBreakVar = true;

		// If we have not reached end of list, go one position ahead
		sockPtr = nextPtr;
	}

	// Send first offline message to the user, if there's at least one offline message
	// for him
	if(!offlineMessagesList.IsEmpty())
		SendNextOfflineMessageForThisUser(msg, m_pDoc, offlineMessagesList);
}



/**************************************************************************/
// Method Name	: SendNextOfflineMessageForThisUser
// Description	: Method to extract one offline message belonging to a user
// Inputs		: Message packet coming from the requester.
//				  Doc pointer
//				  Reference to LOCAL STATIC Offline messages list 
//					(NOT m_OfflineMessagesList in the class)
// Output		: Finds out which is the next msg for the requester from 
//					offline messages list
//				  Sends it to him, if found
//				  Removes the sent msg from the local static list. To ensure this op is
//					Thread safe, we call our owan method RemoveAListMember insted
//					of RemoveAt list member of CListPtr class.
//				  If list is empty (All msgs are sent to the user), it sends a 
//					dummy msg to client with a reason flag of OL. This indicates to
//					the client that it should now show the offline msgs window
// Returns		: void
/**************************************************************************/
void CClientThread::SendNextOfflineMessageForThisUser(CMessagePacket& msg, CMessengerServerDoc *m_pDoc, CPtrList& offlineMessagesList)
{
	if(offlineMessagesList.IsEmpty())
	{
		// Send one dummy message just to inform client that all offline messages 
		// have been sent
		msg.m_strReasonFlag = "OL";
		msg.m_strTo = msg.m_strFrom;
		SendMessageToRecepient(msg, m_pDoc);

		return;
	}

	CMessagePacket *firstMsg = (CMessagePacket *)offlineMessagesList.GetHead();
	POSITION pos = offlineMessagesList.Find(firstMsg);

	firstMsg = (CMessagePacket *)offlineMessagesList.GetNext(pos);

	// Send the message to the user, if it belongs to him/her
	// If this is message from another user, we need to modify the reason flag,
	// so that it appears in the offline messages dialog
	if(firstMsg->m_strReasonFlag == "SM")
		firstMsg->m_strReasonFlag = "O";

	SendMessageToRecepient(*firstMsg, m_pDoc);

	// Remove this entry from list
	offlineMessagesList.RemoveHead();

	// Delete the socket pointed by the current position
	delete(firstMsg);
}



/**************************************************************************/
// Method Name	: ProcessLogoffRequest
// Description	: Method to log off a user
// Inputs		: Message packet coming from the requester.
//				  Socket address from where the msg is coming in
//				  Doc pointer
// Output		: Finds out the client's socket
//				  If this user has logged in
//					Notifies all clients that this user has logged off. Only
//						this user's friends will process that msg. Others will
//						ignore it.
//					Removes the client socket from the logged on clients list. 
//						To ensure this op is Thread safe, we call our owan method 
//						RemoveAListMember insted of RemoveAt list member of CListPtr class.
//				  If user is not logged in, and only connected
//					Removes the client socket from the NOT logged on clients list.
//					Similar care as above is taken for thread safety.
// Returns		: void
/**************************************************************************/
void CClientThread::ProcessLogoffRequest(CMessagePacket &msg, CMessengerClientSocket *pCurrentClient, CMessengerServerDoc *m_pDoc)
{
	// Find out the position of the client's socket. We have to free this socket
	CMessengerClientSocket *clientSockPtr = FindOutTheClient(msg, m_pDoc);

	bool bLoggedInClient = false;

	// If client found
	if(clientSockPtr)
	{
		// This client was logged in and not merely connected
		bLoggedInClient = true;

		// Tell his friends he has logged out
		TellFriendsAbtLogoff(msg, m_pDoc);
	}
	else
	{
		clientSockPtr = pCurrentClient;
	}

	POSITION pos = m_pDoc->m_ClientsList.Find(clientSockPtr);

	if(pos)
	{
		// Remove this socket pointer entry from sockets list
		RemoveAListMember(&(m_pDoc->m_ClientsList), pos, m_pDoc, HANDLE_TO_CLIENTS_LIST_MUTEX);
	}
	// If this position not found, it means this person hasn't logged on
	// In that case the socket can be found in the sockets list
	else
	{
		pos = m_pDoc->m_SocketsList.Find(clientSockPtr);
		RemoveAListMember(&(m_pDoc->m_SocketsList), pos, m_pDoc, HANDLE_TO_SOCKETS_LIST_MUTEX);
	}

	// Free the memory allocated to the client socket
	delete(clientSockPtr);

	// Put this user's name into server's offline users list
	AddUserToList(msg, m_pDoc, OFFLINE_USERS_LIST, bLoggedInClient);

	// Remove the users which are blocked just for this session from the
	// blocked users list
	m_pDoc->RemoveTempBlockedUsers(msg);
}



/**************************************************************************/
// Method Name	: TellFriendsAbtLogoff
// Description	: Method to inform a user's log off to all clients
// Inputs		: Message packet 
//				  Doc pointer
// Output		: Reads thru the whole Logged in clients list
//				  Sends the message to each client that this client has
//					logged off
// Returns		: void
/**************************************************************************/
void CClientThread::TellFriendsAbtLogoff(CMessagePacket& msg, CMessengerServerDoc *m_pDoc)
{
	CMessengerClientSocket *sockPtr = (CMessengerClientSocket *)m_pDoc->m_ClientsList.GetHead();
	CMessengerClientSocket *tailPtr = (CMessengerClientSocket *)m_pDoc->m_ClientsList.GetTail();

	// Store the head position
	POSITION pos = m_pDoc->m_ClientsList.GetHeadPosition();
	sockPtr = (CMessengerClientSocket *)m_pDoc->m_ClientsList.GetNext(pos);

	while(1)
	{
		// Do not tell to self
		if(sockPtr->m_strUserName == msg.m_strFrom)
		{
			if(sockPtr == tailPtr)
				break;

			sockPtr = (CMessengerClientSocket *)m_pDoc->m_ClientsList.GetNext(pos);
			continue;
		}
	
		// Tell about this user logging off to the friends
		msg.m_strTo = sockPtr->m_strUserName;
		msg.m_strReasonFlag = "SX";
		SendMessagePacket(msg, sockPtr, m_pDoc);

		// If we ve reached end of list
		if(sockPtr == tailPtr)
			break;

		// Else go ahead 1 position
		sockPtr = (CMessengerClientSocket *)m_pDoc->m_ClientsList.GetNext(pos);
	}
}



/**************************************************************************/
// Method Name	: AskClientForAddPermission
// Description	: Method to ask a client whether he wants to be added as a friend
//				  of other client
// Inputs		: Message packet to be sent to the client.
//				  Doc pointer
// Output		: Prepares a msg asking for permission
//				  Sends the message to client if he is online
//				  Stores the msg in offline msgs list if he is offline
// Returns		: void
/**************************************************************************/
void CClientThread::AskClientForAddPermission(CMessagePacket& msg, CMessengerServerDoc *m_pDoc)
{
	CMessagePacket *pMsg = new CMessagePacket(msg);

	// Create a msg packet to tell the added friend that he has been added
	CString str = pMsg->m_strMessageBody;
	pMsg->m_strMessageBody = "User " + msg.m_strFrom + " has asked you permission to add you to \r\nhis/her friends list and also sent the following message:\r\n" + str;
	pMsg->m_strReasonFlag = "SFA";

	// Find out if this friend if online
	if(FindOutTheClient(*pMsg, m_pDoc))
	{
		// Tell the added friend that someone is trying to add him/her
		SendMessageToRecepient(*pMsg, m_pDoc);
	}
	else
	{
		// Keep this message as offline message for the To id
		m_pDoc->m_OfflineMessagesList.AddTail(pMsg);
	}
}



/**************************************************************************/
// Method Name	: ProcessAddAFriendRequest
// Description	: Called when a user accepts other user's invitation to be
//					added as a friend
// Inputs		: Message packet to be sent to the client.
//				  Flag - Whether the requested client has accepted invitation
//					and also wants to add sender as his friend / Just accept
//				  Socket address from where the msg is coming in
//				  Doc pointer
// Output		: Prepares a msg saying the kind of permission requested
//					party has given
//				  Loads the User dll to do database ops
//				  According to the type of permission (Accept & Add or Just Add)
//					add the id-friend pair to the database.
//				  Sends the success / failure messages to both clients.
// Returns		: void
/**************************************************************************/
void CClientThread::ProcessAddAFriendRequest(CMessagePacket& msg, BOOL acceptAndAdd, CMessengerClientSocket *pCurrentClient, CMessengerServerDoc *m_pDoc)
{

	CString str = "";
	
	if(acceptAndAdd)
		str = "User " + msg.m_strFrom + " has accepted your add friend request and also added you as his/her friend";
	else
		str = "User " + msg.m_strFrom + " has accepted your add friend request";
		
	SendStatusMessageToClient(str, "FA", msg.m_strTo, pCurrentClient, m_pDoc);

	// Try to add the user using the User dll
	LoadLibrary("User.dll");
	int ret, ret1 = -1;
	
	// Add the client 2 as a friend of requesting party
	ret = addAFriend(msg.m_strTo, msg.m_strFrom);

	// If client 2 also wants to add requester as his friend
	if(acceptAndAdd)
	{
		ret1 = addAFriend(msg.m_strFrom, msg.m_strTo);
	}

	switch(ret1)
	{
	case 0:
		{
			// Tell the sender abt add friend success
			if(FindOutTheClient(msg, m_pDoc))
				SendStatusMessageToClient(msg.m_strTo, "SFN", msg.m_strFrom, pCurrentClient, m_pDoc);
			else
				SendStatusMessageToClient(msg.m_strTo, "SFX", msg.m_strFrom, pCurrentClient, m_pDoc);
		}
		break;
	case 1:
		SendStatusMessageToClient("This name already exists in your friends list", "FF", msg.m_strFrom, pCurrentClient, m_pDoc);
		break;
	case 2:
		SendStatusMessageToClient("Failure adding the Friend", "FF", msg.m_strFrom, pCurrentClient, m_pDoc);
		break;
	case 3:
		SendStatusMessageToClient("This Friend is not a registered user", "FF", msg.m_strFrom, pCurrentClient, m_pDoc);
		break;
	}

	switch(ret)
	{
	case 0:
		{
			// Tell the sender abt add friend success
			if(FindOutTheClient(msg, m_pDoc))
				SendStatusMessageToClient(msg.m_strFrom, "SFN", msg.m_strTo, pCurrentClient, m_pDoc);
			else
				SendStatusMessageToClient(msg.m_strFrom, "SFX", msg.m_strTo, pCurrentClient, m_pDoc);
		}
		break;
	case 1:
		SendStatusMessageToClient("This name already exists in your friends list", "FF", msg.m_strTo, pCurrentClient, m_pDoc);
		break;
	case 2:
		SendStatusMessageToClient("Failure adding the Friend", "FF", msg.m_strTo, pCurrentClient, m_pDoc);
		break;
	case 3:
		SendStatusMessageToClient("This Friend is not a registered user", "FF", msg.m_strTo, pCurrentClient, m_pDoc);
		break;
	}
}



/**************************************************************************/
// Method Name	: ProcessChangePasswordRequest
// Description	: Called when a user sends a password change request
// Inputs		: Message packet to be sent to the client.
//				  Socket address from where the msg is coming in
//				  Doc pointer
// Output		: Loads the User dll to do database ops
//				  Changes the password in database
//				  Sends the success / failure message to client
// Returns		: void
/**************************************************************************/
void CClientThread::ProcessChangePasswordRequest(CMessagePacket& msg, CMessengerClientSocket *pCurrentClient, CMessengerServerDoc *m_pDoc)
{
	// Load the User dll
	LoadLibrary("User.dll");

	// Validate UserName, password
	BOOL ret = changePassword(msg.m_strFrom, msg.m_strMessageBody, msg.m_strTo);

	switch(ret)
	{
	case 0:				// Success
		SendStatusMessageToClient("Password changed successfully", "SC", "", pCurrentClient, m_pDoc);
		break;
	case 1:				// Old password is invalid
		SendStatusMessageToClient("Invalid old password", "FC", "", pCurrentClient, m_pDoc);
		break;
	case 2:				// User not found
		SendStatusMessageToClient("Could not change password, Please try again", "FC", "", pCurrentClient, m_pDoc);
		break;
	case 3:				// Updation failed
		SendStatusMessageToClient("Could not change password due to DB exception, Please try again", "FC", "", pCurrentClient, m_pDoc);
		break;
	}
}



/**************************************************************************/
// Method Name	: RemoveAListMember
// Description	: Method to remove a list member from CPtrList object
//					Depending on the value of whichList, it decides from which 
//					list to remove the member.
//					This method is thread safe!!
// Inputs		: List from where the member is to be removed
//				  Position of the member
//				  Doc pointer
//				  Indicator saying from which list to remove the member
// Output		: Obtains a lock on the list
//				  Removes member
//				  Releases lock on list
// Returns		: void
/**************************************************************************/
// This method removes a member from a CPtrList object. 
void CClientThread::RemoveAListMember(CPtrList *list, POSITION pos, CMessengerServerDoc *m_pDoc, int whichList)
{
	// Find out from which list we want to delete the member. Accordingly we choose the mutex
	// to wait on
	HANDLE hCurrentMutex;

	switch(whichList)
	{
	case HANDLE_TO_SOCKETS_LIST_MUTEX:
		hCurrentMutex = m_pDoc->m_hSocketsListMutex;
		break;
	case HANDLE_TO_CLIENTS_LIST_MUTEX:
		hCurrentMutex = m_pDoc->m_hClientsListMutex;
		break;
	case HANDLE_TO_OFFLINE_MESSAGES_LIST_MUTEX:
		hCurrentMutex = m_pDoc->m_hOfflineMessagesListMutex;
		break;
	}

	// For till other thread releases this list
	::WaitForSingleObject(hCurrentMutex, INFINITE);

	// When the lock is removed, remove the list member
	(*list).RemoveAt(pos);

	// Release this mutex, so that other threads can modify the list
	::ReleaseMutex(hCurrentMutex);
}



/**************************************************************************/
// Method Name	: ProcessDeleteFriendsRequest
// Description	: Method to delete some friends for this user. The friend's 
//					list is emclosed in the stringlist inside message packet
// Inputs		: Message packet coming from the client.
//				  Socket address from where the msg is coming in
//				  Doc pointer
// Output		: Load User dll for database ops
//				  Delete the friend from database
//				  Do this for all the names in the msg list.
//				  Send the results to the user
// Returns		: void
/**************************************************************************/
// 
void CClientThread::ProcessDeleteFriendsRequest(CMessagePacket& msg, CMessengerClientSocket *pCurrentClient, CMessengerServerDoc *m_pDoc)
{
	int status = 0;				// This var will decide the message to be sent back

	// This is a response packet for the client
	CMessagePacket msgPacket;
	msgPacket.m_strFrom = msgPacket.m_strTo = msg.m_strFrom;

	// Load the User dll
	LoadLibrary("User.dll");

	// Get each friend's name from the list and delete it
	while(1)
	{
		if(msg.m_msgList.IsEmpty())
			break;

		// Get the first element out of the list
		CString str = msg.m_msgList.RemoveHead();

		// Remove this friend from the database
		int ret = deleteAFriend(msg.m_strFrom, str);

		switch(ret)
		{
		// If deletion was successful, add this name to the message packet
		case 0:
			msgPacket.m_msgList.AddTail(str);
			break;

		case 1:		//	Deletion unsuccessful
		case 2:		//  DB Exception
			if(!status)		// Change status only if not already changed
				status = ret;
		}

	}

	// Put the appropriate reason flag into the response message packet
	if(!status)			// Everything went well
		msgPacket.m_strReasonFlag = "SDF";
	else				// Some mistake
	{
		msgPacket.m_strReasonFlag = "FDF";

		// Tell client what the problem is
		switch(status)
		{
		case 1:
			msgPacket.m_strMessageBody = "There was some problem in deleting the friends from DB.";
			break;
		case 2:
			msgPacket.m_strMessageBody = "There was a database problem. Please try again later";
			break;
		}
	}

	SendMessagePacket(msgPacket, pCurrentClient, m_pDoc);
}



/**************************************************************************/
// Method Name	: LogMessage
// Description	: Method for logging when a client sends a message packet 
// Inputs		: Message packet sent by client
// Output		: Prints the message log into the log window of server
// Returns		: void
/**************************************************************************/
void CClientThread::LogMessage(CMessagePacket &msg)
{
	CMessengerServerView *view = GetViewPtr();

	// We need to differentiate between msg sent by 1 user to other and msgs
	// sent by user to server
	CString str = msg.m_strTo;
	if(str == "" || msg.m_strReasonFlag == "L" || msg.m_strFrom == msg.m_strTo)
		str = "Server";

	CString strMsg = msg.m_strFrom + " --> " + str + " R = " + msg.m_strReasonFlag + "\r\n";
	view->m_ServerMsgs += strMsg;

	// Show this data on the screen
	view->UpdateData(FALSE);
}



/**************************************************************************/
// Method Name	: AddUserToList
// Description	: Called when a user name is to be moved from one list to another
// Inputs		: Message packet - To field will contain user name.
//				  Doc pointer
//				  An int specifying which list the user should be moved TO.
//				  Indicator whether this user is logged in.
// Output		: Moves the user name from other list control to the list control 
//					indicated by nWhichList
// Returns		: void
/**************************************************************************/
void CClientThread::AddUserToList(CMessagePacket &msg, CMessengerServerDoc *pDoc, int nWhichList, bool bLoggedInClient)
{
	CListCtrl *from, *to;
	CMessengerServerView *view = GetViewPtr();

	// Decide source and target list
	if(nWhichList == ONLINE_USERS_LIST)
	{
		to = &(view->m_lstOnlineUsers);
		from = &(view->m_lstOffLineUsers);
	}
	else
	{
		to = &(view->m_lstOffLineUsers);
		from = &(view->m_lstOnlineUsers);
	}

	// Find the index of given user in the list
	int index = GetListItemIndex(from, msg);

	// Add this to To list
	to->InsertItem(0, from->GetItemText(index, 0));

	// Remove this item from From list
	from->DeleteItem(index);

	// Adjust the offline and online users number
	if(nWhichList == ONLINE_USERS_LIST)
	{
		view->m_nNumOfOfflineUsers --;
		view->m_nNumOfOnlineUsers ++;
	}
	else
	{
		view->m_nNumOfOfflineUsers ++;

		// If this user was logged in decrement num of online users
		if(bLoggedInClient)
			view->m_nNumOfOnlineUsers --;
		view->m_nNumOfConnections --;
	}

	// Show the new values on screen
	view->UpdateData(FALSE);
}



/**************************************************************************/
// Method Name	: GetListItemIndex
// Description	: Used to find position of an item in a list
// Inputs		: List pointer.
//				  Message packet - To field will contain user name.
// Output		: Traverses the list and returns the position of item matching
//					To name in the message packet
// Returns		: position of the matching item if found
//					-1 otherwise
/**************************************************************************/
int CClientThread::GetListItemIndex(CListCtrl *list, CMessagePacket &msg)
{
	// Get count of items. If there are no items in the list, return
	int count = list->GetItemCount();
	if(!count)
		return -1;

	// Traverse the list
	for(int i = 0; i < count; i++)
	{
		// If values match
		CString str = list->GetItemText(i, 0);
		if(str == msg.m_strFrom)
			return i;
	}

	return -1;
}



/**************************************************************************/
// Method Name	: ProcessBlockUserRequest
// Description	: Method for processing Blocking Request
// Inputs		: Message packet sent by the user
//				  Client socket where the msg was received
//				  Doc pointer
//				  A flag indicating whether this user is to be blocked
//					permanently (true) or for this session only (false)
// Output		: Loads user dll
//				  Tries to add the name in To field as the Blocked User
//				  Sends success / failure message to the msg sender.
// Returns		: void
/**************************************************************************/
void CClientThread::ProcessBlockUserRequest(CMessagePacket &msg, CMessengerClientSocket *pCurrentClient, CMessengerServerDoc *m_pDoc, bool bForWhatTime)
{
	// Save the To name first as it's likely to be overwritten
	CString strTo = msg.m_strTo;
	int res = 0;

	// If this user is to be permanently blocked
	if(bForWhatTime)	
	{
		// Load the User dll
		LoadLibrary("User.dll");

		// Add the To user as blocked by From user in the database
		res = addABlockedUser(msg.m_strFrom, msg.m_strTo);

		// Depending on the success status, send ackowledgement to the sender
		CString str;
		switch(res)
		{
		case 0:	// Success
			str = "This user is successfully blocked permanently";
			break;
		case 1: // This User-Blocked user pair already exists
			str = "This user is already blocked by you";
			break;
		case 2:	// Database error
			str = "Database error at server end. Please try again later";
			break;
		case 3:	// Invalid To Id
			str = "This user doesn't exist in the system";
			break;
		}

		// Prepare a message packet to be sent to client
		CMessagePacket msgPacket = msg;
		msgPacket.m_strTo = msg.m_strFrom;	// Person who sent this msg is now receiver
		msgPacket.m_strReasonFlag = "FBU";	// We want to just show a message, so same status for all
		msgPacket.m_strFrom = str;			// Error message

		SendMessageToRecepient(msgPacket, m_pDoc);
	}

	// If d/b op has failed, no need to go further
	if(res)
		return;

	// If this user is blocked for this session only, append a '*' to name
	if(!bForWhatTime)
		strTo += "*";

	// In any case we do the entry in blocked users list here
	// We put all these user names as From names in
	// message packets and put these message packets into each 
	// list element. Each element also contains the owner name
	m_pDoc->AddUserToThisUsersBlockedList(msg.m_strFrom, strTo);
}



/**************************************************************************/
// Method Name	: ProcessMessageFromOneUserToAnother
// Description	: Method for sending a message packet to its intended 
//					recipient
// Inputs		: Message packet sent by the user
//				  Pointer to Socket from where this message is coming 
//				  Doc pointer
// Output		: Checks if the To party has blocked From party
//				  If yes, sends error message to From party
//				  else sends the message to To party
// Returns		: void
/**************************************************************************/
void CClientThread::ProcessMessageFromOneUserToAnother(CMessagePacket& msg, CMessengerClientSocket *pCurrentClient, CMessengerServerDoc *pDoc)
{
	// Find out if Recipient (To party) has blocked From party.
	if(IsThisFromPartyBlocked(msg, pDoc))
	{
		msg.m_strReasonFlag = "FM";
		CString str = msg.m_strTo;	// Store the To name first
		msg.m_strTo = msg.m_strFrom;	// We are sending msg back to From party
		msg.m_strFrom = "The user " + str + " has blocked you, so message could not be delivered";
	}
	else
	{
		// Attach a success flag to the message packet
		msg.m_strReasonFlag = "SM";
		strcpy(msg.m_MessageFont.lfFaceName, msg.m_strFontFace.GetBuffer(0));
	}

	// Get the client socket ptr for To party
	SendMessageToRecepient(msg, pDoc);
}



/**************************************************************************/
// Method Name	: IsThisFromPartyBlocked
// Description	: Checks whether the To party in the message packet has
//					blocked the From party
// Inputs		: Message packet sent by the user
//				  Doc pointer
// Output		: Checks if the To party has blocked From party
// Returns		: true - To party has blocked From party
//				  false - otherwise
/**************************************************************************/
bool CClientThread::IsThisFromPartyBlocked(CMessagePacket &msg, CMessengerServerDoc *pDoc)
{	
	///////////////////////////////////////////////////
	// First find the element whose owner is To party
	///////////////////////////////////////////////////

	// If list is empty, no need to go further
	if(pDoc->m_BlockedUsersList.IsEmpty())
		return false;

	// Get addresses of head and tail members of the list
	CBlockedUserListElement *pHeadElement = (CBlockedUserListElement *)pDoc->m_BlockedUsersList.GetHead();
	CBlockedUserListElement *pTailElement = (CBlockedUserListElement *)pDoc->m_BlockedUsersList.GetTail();
	CBlockedUserListElement *pNextElement = NULL;

	// Store the head position
	POSITION pos = pDoc->m_BlockedUsersList.GetHeadPosition();
	pHeadElement = (CBlockedUserListElement *)pDoc->m_BlockedUsersList.GetNext(pos);

	while(1)
	{
		if(pHeadElement != pTailElement)
			pNextElement = (CBlockedUserListElement *)pDoc->m_BlockedUsersList.GetNext(pos);

		// If this current element is the required element
		// Search within that element for From party's name
		if(pHeadElement->m_strUser == msg.m_strTo)
		{
			///////////////////////////////////////////////////
			// Now find the From party name within that element
			///////////////////////////////////////////////////

			// If there are no blocked users entries
			if(pHeadElement->m_BlockedUsers.IsEmpty())
				return false;

			// Get addresses of head and tail members of the list
			CMessagePacket *pHeadMsg = (CMessagePacket *)pHeadElement->m_BlockedUsers.GetHead();
			CMessagePacket *pTailMsg = (CMessagePacket *)pHeadElement->m_BlockedUsers.GetTail();
			CMessagePacket *pNextMsg = NULL;

			// Store the head position
			POSITION pos = pHeadElement->m_BlockedUsers.GetHeadPosition();
			pHeadMsg = (CMessagePacket *)pHeadElement->m_BlockedUsers.GetNext(pos);

			while(1)
			{
				// Store next position. Don't do this if there's only 1 element. Otherwise
				// Server will crash!
				if(pHeadMsg != pTailMsg)
					pNextMsg = (CMessagePacket *)pHeadElement->m_BlockedUsers.GetNext(pos);

				// Before comparing, remove the '*' char, if any, from blocked user's name
				// This '*' signifies this user is blocked only for current session
				CString str = pHeadMsg->m_strFrom;
				str.Remove('*');

				// If From party's name found in blocked users list
				if(str == msg.m_strFrom)
					return true;

				// If we ve reached end of list
				if(pHeadMsg == pTailMsg)
					break;

				// Else go ahead 1 position
				pHeadMsg = pNextMsg;
			}
			// We have reached here without finding anything, so return false
			return false;
		}

		// If we ve reached end of list
		if(pHeadElement == pTailElement)
			break;

		// Else go ahead 1 position
		pHeadElement = pNextElement;
	}

	// We have reached here without finding anything, so return false
	return false;
}



/**************************************************************************/
// Method Name	: SendBlockUserList
// Description	: Sends blocked users list to msg sender
// Inputs		: Message packet sent by the user
//				  Socket from where msg came in
//				  Doc pointer
// Output		: 
// Returns		: void
/**************************************************************************/
void CClientThread::SendBlockUserList(CMessagePacket &msg, CMessengerClientSocket *pCurrentClient, CMessengerServerDoc *pDoc)
{
	// First find the list of which this user is owner
	CBlockedUserListElement *element = FindBlockedUserElement(msg.m_strFrom, pDoc);

	// Modify the message packet to send to client
	msg.m_strReasonFlag = "BU";		// Blocked users list

	// Put the blocked users list into the string list inside msg packet
	msg.m_msgList.RemoveAll();		// Clear the msg list first

	// Traverse the element and transfer
	if((element) && (!element->m_BlockedUsers.IsEmpty()))
	{
		// Get addresses of head and tail members of the list
		CMessagePacket *pHeadMsg = (CMessagePacket *)element->m_BlockedUsers.GetHead();
		CMessagePacket *pTailMsg = (CMessagePacket *)element->m_BlockedUsers.GetTail();
		CMessagePacket *pNextMsg = NULL;

		// Store the head position
		POSITION pos = element->m_BlockedUsers.GetHeadPosition();
		pHeadMsg = (CMessagePacket *)element->m_BlockedUsers.GetNext(pos);

		while(1)
		{
			// Store next position. Don't do this if there's only 1 element. Otherwise
			// Server will crash!
			if(pHeadMsg != pTailMsg)
				pNextMsg = (CMessagePacket *)element->m_BlockedUsers.GetNext(pos);

			// Put this name into the string list
			msg.m_msgList.AddTail(pHeadMsg->m_strFrom);

			// If we ve reached end of list
			if(pHeadMsg == pTailMsg)
				break;

			// Else go ahead 1 position
			pHeadMsg = pNextMsg;
		}
	}

	// Send acknowledgement to client
	pDoc->SendMessagePacket(msg, pCurrentClient);
}



/**************************************************************************/
// Method Name	: FindBlockedUserElement
// Description	: Finds the address of element from Blocked Users list whose 
//					owner is the sender.
// Inputs		: Owner name
//				  doc pointer
// Output		: None
// Returns		: Address of blocked users element if found
//				  NULL otherwise
/**************************************************************************/
CBlockedUserListElement *CClientThread::FindBlockedUserElement(CString owner, CMessengerServerDoc *pDoc)
{
	// If list is empty, no need to go further
	if(pDoc->m_BlockedUsersList.IsEmpty())
		return NULL;

	// Get addresses of head and tail members of the list
	CBlockedUserListElement *pHeadElement = (CBlockedUserListElement *)pDoc->m_BlockedUsersList.GetHead();
	CBlockedUserListElement *pTailElement = (CBlockedUserListElement *)pDoc->m_BlockedUsersList.GetTail();
	CBlockedUserListElement *pNextElement = NULL;

	// Store the head position
	POSITION pos = pDoc->m_BlockedUsersList.GetHeadPosition();
	pHeadElement = (CBlockedUserListElement *)pDoc->m_BlockedUsersList.GetNext(pos);

	while(1)
	{
		if(pHeadElement != pTailElement)
			pNextElement = (CBlockedUserListElement *)pDoc->m_BlockedUsersList.GetNext(pos);

		// If this current element is the required element
		// return this pointer
		if(pHeadElement->m_strUser == owner)
			return pHeadElement;

		// If we ve reached end of list
		if(pHeadElement == pTailElement)
			break;

		// Else go ahead 1 position
		pHeadElement = pNextElement;
	}

	// We reached end of list without finding anything, so return NULL
	return NULL;
}



/**************************************************************************/
// Method Name	: RemoveSelBlockedUsers
// Description	: Removes some users from blocked users list
// Inputs		: Msg packet sent by the owner of blocked users list
//				  Socket where msg packet was received
//				  Doc pointer
//				  doc pointer
// Output		: Finds out the blocked list element belonging to the 
//					msg packet sender
//				  Traverses all user name under that element
//				  Deletes those matching the sent names
//				  Sends acknowledgement to the client
// Returns		: void
/**************************************************************************/
void CClientThread::RemoveSelBlockedUsers(CMessagePacket &msg, CMessengerClientSocket *pCurrentClient, CMessengerServerDoc *pDoc)
{
	// Find the address of the sender's element
	CBlockedUserListElement *element = FindBlockedUserElement(msg.m_strFrom, pDoc);

	// Modify the msg packet
	msg.m_strTo = msg.m_strFrom;
	msg.m_strReasonFlag = "FBUR";		// Blocked users edit failed (by default)
	CString err = "Failed to edit blocked users list. Please re-login to get correct status";

	// If element found and has a blocked users list
	if(element || element->m_BlockedUsers.IsEmpty())
	{
		msg.m_strReasonFlag = "SBUR";		// Blocked users edit success

		// First get all the strings out of msg packet
		int cnt = msg.m_msgList.GetCount();

		// Create this array to hold all strings
		CString *strArr = new CString[cnt];

		// Fill the array
		for(int i = 0; i < cnt; i++)
		{
			CString str = msg.m_msgList.RemoveHead();
			strArr[i] = str;
		}

		// Store initial head and tail positions of bloked users list
		CMessagePacket *msgBlockedHead = (CMessagePacket *)element->m_BlockedUsers.GetHead();
		CMessagePacket *msgBlockedTail = (CMessagePacket *)element->m_BlockedUsers.GetTail();
		CMessagePacket *msgBlockedNext = NULL;
	
		// Store the head position
		POSITION pos = element->m_BlockedUsers.GetHeadPosition();
		POSITION initPos = pos;
		msgBlockedHead = (CMessagePacket *)element->m_BlockedUsers.GetNext(pos);

		while(1)
		{
			// Store next position
			if(msgBlockedHead != msgBlockedTail)
			{
				// Store the initial position
				initPos = pos;

				// Store the pointer to next element
				msgBlockedNext = (CMessagePacket *)element->m_BlockedUsers.GetNext(pos);
			}

			// Compare this msg packet's from field with each element of string array
			for(int i = 0; i < cnt; i++)
			{
				// If found 
				if(msgBlockedHead->m_strFrom == strArr[i])
				{
					// If a permanently blocked user
					if(msgBlockedHead->m_strFrom.Find('*') == -1)
					{
						// Remove this user as blocked user from database
						// Load the User dll
						LoadLibrary("User.dll");

						// Remove this username, blocked user name pair from database
						int ret = deleteABlockedUser(msg.m_strFrom, strArr[i]);

						if(ret)
						{
							// If blocked users edit failed, send msg and get out
							msg.m_strReasonFlag = "FBUR";
							SendStatusMessageToClient(err, msg.m_strReasonFlag, msg.m_strTo, pCurrentClient, pDoc);
							return;
						}
						
					}

					// Find the position of element to be deleted
					POSITION delPos = element->m_BlockedUsers.Find(msgBlockedHead);

					// Remove this user as blocked user from memory
					element->m_BlockedUsers.RemoveAt(delPos);
					delete msgBlockedHead;

					// Since now we found the desired element and deleted it, get out
					break;
				}
			}

			// If we ve reached end of list
			if(msgBlockedHead == msgBlockedTail)
				break;

			// Go ahead 1 position
			msgBlockedHead = msgBlockedNext;
		}

		// Free the memory used for storing strings
		delete[] strArr;
	}

	// Send the reply to the client
	err = "Blocked users removal successful";
	SendStatusMessageToClient(err, msg.m_strReasonFlag, msg.m_strTo, pCurrentClient, pDoc);
}
