//////////////////////////////////////////////////////////////////////
// ClientThread.h:
// interface for the CClientThread class.
// The key class for all the functionality of server. This class contains
// many key methods for interacting with a client.
// Whenever a client connects, one object of this class is created. All the
// client interaction takes place thru this object.
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_CLIENTTHREAD_H__9308BE46_6EF8_11D6_8144_0001031200E7__INCLUDED_)
#define AFX_CLIENTTHREAD_H__9308BE46_6EF8_11D6_8144_0001031200E7__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
// ClientThread.h : header file
//

#include "MessengerServerDoc.h"
#include "MessengerServerSocket.h"
#include "MessengerClientSocket.h"
#include "MessagePacket.h"
#include "UserExported.h"
#include "MessengerClientSocket.h"
#include "UserRecord.h"
#include "MessengerServerView.h"
#include "MainFrm.h"
#include "BlockedUserListElement.h"
#include "BlockedUserRecord.h"

/////////////////////////////////////////////////////////////////////////////
// CClientThread thread

class CClientThread : public CWinThread
{
	DECLARE_DYNCREATE(CClientThread)
protected:
	CClientThread();           // protected constructor used by dynamic creation

// Attributes
public:

// Operations
public:

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CClientThread)
	public:
	virtual BOOL InitInstance();
	virtual int ExitInstance();
	virtual int Run();
	//}}AFX_VIRTUAL

// Implementation
protected:
	void RemoveSelBlockedUsers(CMessagePacket &msg, CMessengerClientSocket *pCurrentClient, CMessengerServerDoc *pDoc);
	CBlockedUserListElement *FindBlockedUserElement(CString owner, CMessengerServerDoc *pDoc);
	void SendBlockUserList(CMessagePacket& msg, CMessengerClientSocket *pCurrentClient, CMessengerServerDoc *pDoc);
	bool IsThisFromPartyBlocked(CMessagePacket& msg, CMessengerServerDoc *pDoc);
	void ProcessMessageFromOneUserToAnother(CMessagePacket& msg, CMessengerClientSocket *pCurrentClient, CMessengerServerDoc *pDoc);
	void ProcessBlockUserRequest(CMessagePacket& msg, CMessengerClientSocket *pCurrentClient, CMessengerServerDoc *m_pDoc, bool bForWhatTime);
	int GetListItemIndex(CListCtrl *list, CMessagePacket &msg);
	void AddUserToList(CMessagePacket& msg, CMessengerServerDoc *pDoc, int nWhichList, bool bLoggedInClient);
	void LogMessage(CMessagePacket& msg);
	virtual ~CClientThread();

	// Generated message map functions
	//{{AFX_MSG(CClientThread)
		// NOTE - the ClassWizard will add and remove member functions here.
	//}}AFX_MSG

	DECLARE_MESSAGE_MAP()
public:
	void ProcessDeleteFriendsRequest(CMessagePacket& msg, CMessengerClientSocket *pCurrentClient, CMessengerServerDoc *m_pDoc);
	bool ReceiveMessagePacket(CMessengerClientSocket *pCurrentClient);
	bool ProcessMessagePacket(CMessagePacket& msg, CMessengerClientSocket *pCurrentClient);
	CMessengerServerView *GetViewPtr();
	void ProcessLoginRequest(CMessagePacket& msg, CMessengerClientSocket *pCurrentClient, CMessengerServerDoc *m_pDoc);
	bool SendStatusMessageToClient(CString msg, CString status, CString strRecipient, CMessengerClientSocket *sock, CMessengerServerDoc *m_pDoc);
	bool SendMessagePacket(CMessagePacket& msgPacket, CMessengerClientSocket *clientSocket, CMessengerServerDoc *m_pDoc);
	static CMessengerClientSocket *FindOutTheClient(CMessagePacket& msg, CMessengerServerDoc *m_pDoc);
	CMessengerClientSocket *RemoveFromSocketsList(CMessengerClientSocket *socket, CMessengerServerDoc *m_pDoc);
	void TellFriendsAbtLogin(CMessagePacket& msg, CMessengerServerDoc *m_pDoc);
	void ProcessRegistrationRequest(CMessagePacket& msg, CMessengerClientSocket *pCurrentClient, CMessengerServerDoc *m_pDoc);
	void SendMessageToRecepient(CMessagePacket& msg, CMessengerServerDoc *m_pDoc);
	void SendListOfFriendsToThisLogin(CMessagePacket& msg, CMessengerServerDoc *m_pDoc);
	void GetListOfAllFriends(CMessagePacket& msg, CStringList& friendsList, CMessengerServerDoc *m_pDoc);
	void GetAllOfflineMessagesForThisUser(CMessagePacket& msg, CMessengerServerDoc *m_pDoc, bool isGettingMessages);
	void SendNextOfflineMessageForThisUser(CMessagePacket& msg, CMessengerServerDoc *m_pDoc, CPtrList& offlineMessagesList);
	void ProcessLogoffRequest(CMessagePacket &msg, CMessengerClientSocket *pCurrentClient, CMessengerServerDoc *m_pDoc);
	void TellFriendsAbtLogoff(CMessagePacket& msg, CMessengerServerDoc *m_pDoc);
	void AskClientForAddPermission(CMessagePacket& msg, CMessengerServerDoc *m_pDoc);
	void ProcessAddAFriendRequest(CMessagePacket& msg, BOOL acceptAndAdd, CMessengerClientSocket *pCurrentClient, CMessengerServerDoc *m_pDoc);
	void ProcessChangePasswordRequest(CMessagePacket& msg, CMessengerClientSocket *pCurrentClient, CMessengerServerDoc *m_pDoc);
	void RemoveAListMember(CPtrList *list, POSITION pos, CMessengerServerDoc *m_pDoc, int whichList);
};

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_CLIENTTHREAD_H__9308BE46_6EF8_11D6_8144_0001031200E7__INCLUDED_)
