// MessengerServerDoc.cpp : implementation of the CMessengerServerDoc class
//

#include "stdafx.h"
#include "MessengerServer.h"

#include "MessengerServerDoc.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#include "GlobalThread.h"

/////////////////////////////////////////////////////////////////////////////
// CMessengerServerDoc

IMPLEMENT_DYNCREATE(CMessengerServerDoc, CDocument)

BEGIN_MESSAGE_MAP(CMessengerServerDoc, CDocument)
	//{{AFX_MSG_MAP(CMessengerServerDoc)
		// NOTE - the ClassWizard will add and remove mapping macros here.
		//    DO NOT EDIT what you see in these blocks of generated code!
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMessengerServerDoc construction/destruction

CMessengerServerDoc::CMessengerServerDoc()
{
	m_pServerSocket = NULL;
	m_pClientSocket = NULL;

	// Create mutexes for various shared resources
	m_hClientsListMutex = ::CreateMutex(NULL, FALSE, NULL);
	m_hSocketsListMutex = ::CreateMutex(NULL, FALSE, NULL);
	m_hOfflineMessagesListMutex = ::CreateMutex(NULL, FALSE, NULL);
}

CMessengerServerDoc::~CMessengerServerDoc()
{
	if(m_pServerSocket)
		delete(m_pServerSocket);
//	if(m_pClientSocket)
//		delete(m_pClientSocket);

	m_ClientsList.RemoveAll();
	m_OfflineMessagesList.RemoveAll();
	m_SocketsList.RemoveAll();
}



/**************************************************************************/
// Method Name	: OnNewDocument
// Description	: System defined method, called when new doc is created
// Inputs		: None
// Output		: Calls base class method
//				  Sets the title to 'Messenger Server'
//				  Reads config settings from config file
//				  Creates a server socket and starts listening on it
// Returns		: TRUE - Success
//				  FALSE - Otherwise
/**************************************************************************/
BOOL CMessengerServerDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;

	// Set the title of the document
	SetTitle("Chatter-jee Server");

	// Get the port num, ip address data from config file
	if(!GetConfigData())
	{
		::AfxMessageBox("Configuration file not found or corrupt. Please get the correct configuration file Config.txt", MB_OK|MB_ICONSTOP);
		return FALSE;
	}

	// Create the list of users blocked by the logging in user
	CreateBlockedUsersList();

	// Create a serverside socket
	m_pServerSocket = new CMessengerServerSocket(this);

	if(m_pServerSocket->Create(m_nPortNum))
	{
		// Start Listening
		if(m_pServerSocket->Listen())
			return TRUE;
	}

	return FALSE;
}



/////////////////////////////////////////////////////////////////////////////
// CMessengerServerDoc serialization

void CMessengerServerDoc::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
		// TODO: add storing code here
	}
	else
	{
		// TODO: add loading code here
	}
}

/////////////////////////////////////////////////////////////////////////////
// CMessengerServerDoc diagnostics

#ifdef _DEBUG
void CMessengerServerDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CMessengerServerDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMessengerServerDoc commands

/**************************************************************************/
// Method Name	: processAccept
// Description	: Called when connection request is made by a client
// Inputs		: None
// Output		: Creates a client socket 
//				  Creates socket file and i-o archives
//				  Creates and starts a client thread
// Returns		: 0 - Success
//				  -1 - Otherwise
/**************************************************************************/
int CMessengerServerDoc::processAccept()
{
	// Create a new client socket to accept connection
	m_pClientSocket = new CMessengerClientSocket(this);

	// Accept conenction on this socket
	if(m_pServerSocket->Accept(*m_pClientSocket))
	{
		TRY
		{
		// Construct file object
		m_pClientSocket->m_pSocketFile = new CSocketFile(m_pClientSocket);
	
		// Construct archive objects for i-o
		m_pClientSocket->m_pArIn = new CArchive(m_pClientSocket->m_pSocketFile, CArchive::load);
		m_pClientSocket->m_pArOut = new CArchive(m_pClientSocket->m_pSocketFile, CArchive::store);

		// Add this connection to the sockets list
		m_SocketsList.AddTail(m_pClientSocket);

		// Create a new thread for this client
		CClientThread *clientThread = (CClientThread *)::AfxBeginThread(ThreadProc, this);
		m_pClientSocket->m_pParentThread = clientThread;

		// Notify to the view abt new connection added
		CMessengerServerView *view = GetViewPtr();
		view->m_nNumOfConnections ++;
		view->UpdateData(FALSE);
		}
		CATCH(CException, e)
		{
			TRACE("\nCaught!!\n");
			return -1;
		}
		END_CATCH
	}

	return 0;
}



/**************************************************************************/
// Method Name	: SendMessagePacket()
// Description	: Sends 1 message packet to the client
// Inputs		: Message packet
// Output		: Sends message packet thru output archive
// Returns		: TRUE if successful, otherwise FALSE
/**************************************************************************/
bool CMessengerServerDoc::SendMessagePacket(CMessagePacket& msgPacket, CMessengerClientSocket *clientSocket)
{
	TRY
	{
		msgPacket.Serialize(*clientSocket->m_pArOut);
		clientSocket->m_pArOut->Flush();

		// TEMP
		TRACE("\nMessage sent thru archive\n");
	}
	CATCH(CFileException, e)
	{
		return FALSE;
	}
	END_CATCH

	return TRUE;
}



/**************************************************************************/
// Method Name	: SaveModified()
// Description	: Called by system when server program is closing
// Inputs		: None
// Output		: Sends notification to all logged in clients that server
//					is shutting down
//				  Clears memory occupied by all lists
//				  Calls base class method
// Returns		: TRUE if successful, otherwise FALSE
/**************************************************************************/
BOOL CMessengerServerDoc::SaveModified() 
{
	// Send the notification to all the clients online
	SendServerClosingNotificationToAllClients();

	// Clear off the sockets list and offline messages list
	EmptyAllLists();

	return CDocument::SaveModified();
}



/**************************************************************************/
// Method Name	: EmptyAllLists()
// Description	: Empties all list objects
// Inputs		: None
// Output		: Calls other methods to empty all lists and clear the 
//					memory used by list objects.
// Returns		: void
// Note			: Since many threads will be using the lists, locks are obtained
//				  on lists when they are to be freed. This makes this method thread
//				  safe
/**************************************************************************/
void CMessengerServerDoc::EmptyAllLists()
{
	// FREE THE SOCKETS LIST
	// Wait till other threads release this list
	::WaitForSingleObject(m_hSocketsListMutex, INFINITE);
	// When the lock is removed, remove the list members
	EmptySocketsList();
	// Release this mutex, so that other threads can modify the list
	::ReleaseMutex(m_hSocketsListMutex);

	// FREE THE CLIENTS LIST
	// Wait till other threads release this list
	::WaitForSingleObject(m_hClientsListMutex, INFINITE);
	// When the lock is removed, remove the list members
	EmptyClientsList();
	// Release this mutex, so that other threads can modify the list
	::ReleaseMutex(m_hClientsListMutex);

	// FREE THE OFFLINE MESSAGES LIST
	// Wait till other threads release this list
	::WaitForSingleObject(m_hOfflineMessagesListMutex, INFINITE);
	// When the lock is removed, remove the list members
	EmptyOfflineMessagesList();
	// Release this mutex, so that other threads can modify the list
	::ReleaseMutex(m_hOfflineMessagesListMutex);

	// FREE THE BLOCKED USERS LIST
	// Wait till other threads release this list
	::WaitForSingleObject(m_hBlockedUsersListMutex, INFINITE);
	// When the lock is removed, remove the list members
	EmptyBlockedUsersList();
	// Release this mutex, so that other threads can modify the list
	::ReleaseMutex(m_hBlockedUsersListMutex);
}



/**************************************************************************/
// Method Name	: EmptyClientsList()
// Description	: Empties logged in users list
// Inputs		: None
// Output		: Empties logged in users' list and clears the 
//					memory used by that list's objects.
// Returns		: void
/**************************************************************************/
void CMessengerServerDoc::EmptyClientsList()
{
	if(m_ClientsList.IsEmpty())
		return;

	// Empty the socket clients list
	CMessengerClientSocket *sockPtr = (CMessengerClientSocket *)m_ClientsList.GetHead();
	CMessengerClientSocket *tailPtr = (CMessengerClientSocket *)m_ClientsList.GetTail();
	CMessengerClientSocket *nextPtr = NULL;

	POSITION pos = m_ClientsList.Find(sockPtr);

	sockPtr = (CMessengerClientSocket *)m_ClientsList.GetNext(pos);

	while(1)
	{
		// If this is not the last entry in the list, record the next position
		if(sockPtr != tailPtr)
		{
			nextPtr = (CMessengerClientSocket *)m_ClientsList.GetNext(pos);
		}

		// Delete the socket pointed by the current position
		delete(sockPtr);

		// If we are at the end of list, get out
		if(sockPtr == tailPtr)
			break;

		// If we have not reached end of list, go one position ahead
		sockPtr = nextPtr;
	}
}



/**************************************************************************/
// Method Name	: EmptyOfflineMessagesList()
// Description	: Empties offline messages list
// Inputs		: None
// Output		: Empties logged in offline messages list and clears the 
//					memory used by that list's objects.
// Returns		: void
/**************************************************************************/
void CMessengerServerDoc::EmptyOfflineMessagesList()
{
	if(m_OfflineMessagesList.IsEmpty())
		return;

	// Empty the socket clients list
	CMessagePacket *sockPtr = (CMessagePacket *)m_OfflineMessagesList.GetHead();
	CMessagePacket *tailPtr = (CMessagePacket *)m_OfflineMessagesList.GetTail();
	CMessagePacket *nextPtr = NULL;

	POSITION pos = m_OfflineMessagesList.Find(sockPtr);

	sockPtr = (CMessagePacket *)m_OfflineMessagesList.GetNext(pos);

	while(1)
	{
		// If this is not the last entry in the list, record the next position
		if(sockPtr != tailPtr)
		{
			nextPtr = (CMessagePacket *)m_OfflineMessagesList.GetNext(pos);
		}

		// Delete the socket pointed by the current position
		delete(sockPtr);

		// If we are at the end of list, get out
		if(sockPtr == tailPtr)
			break;

		// If we have not reached end of list, go one position ahead
		sockPtr = nextPtr;
	}
}



/**************************************************************************/
// Method Name	: EmptySocketsList()
// Description	: Empties not logged in users list
// Inputs		: None
// Output		: Empties logged in not logged in users list and clears the 
//					memory used by that list's objects.
// Returns		: void
/**************************************************************************/
void CMessengerServerDoc::EmptySocketsList()
{
	if(m_SocketsList.IsEmpty())
		return;

	// Empty the socket clients list
	CMessengerClientSocket *sockPtr = (CMessengerClientSocket *)m_SocketsList.GetHead();
	CMessengerClientSocket *tailPtr = (CMessengerClientSocket *)m_SocketsList.GetTail();
	CMessengerClientSocket *nextPtr = NULL;

	POSITION pos = m_SocketsList.Find(sockPtr);

	sockPtr = (CMessengerClientSocket *)m_SocketsList.GetNext(pos);

	while(1)
	{
		// If this is not the last entry in the list, record the next position
		if(sockPtr != tailPtr)
		{
			nextPtr = (CMessengerClientSocket *)m_SocketsList.GetNext(pos);
		}

		// Delete the socket pointed by the current position
		delete(sockPtr);

		// If we are at the end of list, get out
		if(sockPtr == tailPtr)
			break;

		// If we have not reached end of list, go one position ahead
		sockPtr = nextPtr;
	}
}


/**************************************************************************/
// Method Name	: EmptyBlockedUsersList
// Description	: Method for cleaning Blocked users list
// Inputs		: None
// Output		: Deletes each object pointed to by blocked users list
//				  Empties the blocked users list
// Returns		: void
/**************************************************************************/
void CMessengerServerDoc::EmptyBlockedUsersList()
{
	// If list is empty, no need to go further
	if(m_BlockedUsersList.IsEmpty())
		return;

	// Get addresses of head and tail members of the list
	CBlockedUserListElement *pHeadElement = (CBlockedUserListElement *)m_BlockedUsersList.GetHead();
	CBlockedUserListElement *pTailElement = (CBlockedUserListElement *)m_BlockedUsersList.GetTail();
	CBlockedUserListElement *pNextElement = NULL;

	// Store the head position
	POSITION pos = m_BlockedUsersList.GetHeadPosition();
	pHeadElement = (CBlockedUserListElement *)m_BlockedUsersList.GetNext(pos);

	while(1)
	{
		if(pHeadElement != pTailElement)
			pNextElement = (CBlockedUserListElement *)m_BlockedUsersList.GetNext(pos);

		// Empty the list under this element before deleting it
		EmptyBlockedUsersListElement(pHeadElement);

		// Delete the memory pointed to by pHeadElement
		delete(pHeadElement);
		
		// If we ve reached end of list
		if(pHeadElement == pTailElement)
			break;

		// Else go ahead 1 position
		pHeadElement = pNextElement;
	}
}



/**************************************************************************/
// Method Name	: EmptyBlockedUsersListElement
// Description	: Method for cleaning Blocked users list element
// Inputs		: None
// Output		: Deletes each object pointed to by the list inside an element
//				  Empties the list
// Returns		: void
/**************************************************************************/
void CMessengerServerDoc::EmptyBlockedUsersListElement(CBlockedUserListElement *element)
{
	if(element->m_BlockedUsers.IsEmpty())
		return;

	// Get addresses of head and tail members of the list
	CMessagePacket *pHeadMsg = (CMessagePacket *)element->m_BlockedUsers.GetHead();
	CMessagePacket *pTailMsg = (CMessagePacket *)element->m_BlockedUsers.GetTail();
	CMessagePacket *pNextMsg = NULL;

	// Store the head position
	POSITION pos = element->m_BlockedUsers.GetHeadPosition();
	pHeadMsg = (CMessagePacket *)element->m_BlockedUsers.GetNext(pos);

	while(1)
	{
		if(pHeadMsg != pTailMsg)
			pNextMsg = (CMessagePacket *)element->m_BlockedUsers.GetNext(pos);

		// Delete the memory pointed to by pHeadMsg
		delete(pHeadMsg);
		
		// If we ve reached end of list
		if(pHeadMsg == pTailMsg)
			break;

		// Else go ahead 1 position
		pHeadMsg = pNextMsg;
	}
}



/**************************************************************************/
// Method Name	: SendServerClosingNotificationToAllClients()
// Description	: Notifies all clients that server is shutting down
// Inputs		: None
// Output		: Calls other methods to notify all logged in users and then
//					all users thet are connected but not logged in
// Returns		: void
/**************************************************************************/
void CMessengerServerDoc::SendServerClosingNotificationToAllClients()
{
	// Send notification to logged on clients
	SendServerClosingNotificationToListMembers(m_ClientsList);

	// Send notification to clients which are connected but not logged on
	SendServerClosingNotificationToListMembers(m_SocketsList);
}



/**************************************************************************/
// Method Name	: SendServerClosingNotificationToListMembers()
// Description	: Notifies all clients in the given list that server is 
//				  shutting down
// Inputs		: List of users to whom msg is to be sent
// Output		: Message is sent to all list members
// Returns		: void
/**************************************************************************/
void CMessengerServerDoc::SendServerClosingNotificationToListMembers(CPtrList &list)
{
	CMessagePacket msg;
	msg.m_strReasonFlag = "Z";		// Indicates server is shutting down

	if(list.IsEmpty())
		return;

	// Empty the socket clients list
	CMessengerClientSocket *sockPtr = (CMessengerClientSocket *)list.GetHead();
	CMessengerClientSocket *tailPtr = (CMessengerClientSocket *)list.GetTail();

	POSITION pos = list.Find(sockPtr);

	sockPtr = (CMessengerClientSocket *)list.GetNext(pos);

	while(1)
	{
		// Send this message to the client
		SendMessagePacket(msg, sockPtr);

		// If we are at the end of list, get out
		if(sockPtr == tailPtr)
			break;

		// Otherwise go ahead 1 position
		sockPtr = (CMessengerClientSocket *)list.GetNext(pos);
	}
}



/**************************************************************************/
// Method Name	: GetConfigData()
// Description	: Reads config data from config file
// Inputs		: None
// Output		: Opens config file
//				  Reads port num into doc vars
// Returns		: true - Success
//				  false - Otherwise
/**************************************************************************/
bool CMessengerServerDoc::GetConfigData()
{	
	ifstream sourceHandle;			// Handle to config file

	// Open the GLOW log. Don't create it if it wasn't there.
	sourceHandle.open("config.txt", ios::nocreate);

	if(!sourceHandle)
		return false;

	char line[CONFIG_FILE_LINE_LENGTH];

	// Copy glow log file to our log file
	while(1)
	{
		// Get a single line from the file
		sourceHandle.getline(line, CONFIG_FILE_LINE_LENGTH);

		// If we have reached the end of file, this last line is useless,
		// so ignore it
		if(sourceHandle.eof())
			break;

		// If this is a comment line, ignore it
		if((line[0] == line[1]) && line[0] == '/')
			continue;

		// Break the line and fill up the config data
		char *key = strtok(line, "=");
		char *value = &line[strlen(key) + 1];

		if(!strcmp(key, "PORT_NUM"))
			m_nPortNum = atoi(value);
	}

	// Close the files
	sourceHandle.close();

	return true;
}



/**************************************************************************/
// Method Name	: GetViewPtr
// Description	: Method for finding view pointer
// Inputs		: None
// Output		: None
// Returns		: view pointer
/**************************************************************************/
CMessengerServerView *CMessengerServerDoc::GetViewPtr()
{
	CMainFrame *wnd = (CMainFrame *)::AfxGetMainWnd();

	CMessengerServerView *pView = (CMessengerServerView *)wnd->GetActiveView();

	return pView;
}



/**************************************************************************/
// Method Name	: AddUserToThisUsersBlockedList
// Description	: Method for adding a user to blocked user's list
// Inputs		: User name who is the owner of blocked users list
//				  User whose name is to be added in blocked users list
// Output		: Searches thru the Blocked users list for input name
//				  If input name not found, creates a blocked users list element
//				  Adds the input name as one of the blocked users under that list
// Returns		: void
/**************************************************************************/
void CMessengerServerDoc::AddUserToThisUsersBlockedList(CString& ownerName, CString& userName)
{
	// If list is empty, directly create a new element and add it
	if(m_BlockedUsersList.IsEmpty())
	{
		// Create an element and return
		CreateBlockedUserElement(ownerName, userName);
		return;
	}

	// Get addresses of head and tail members of the list
	CBlockedUserListElement *pHeadElement = (CBlockedUserListElement *)m_BlockedUsersList.GetHead();
	CBlockedUserListElement *pTailElement = (CBlockedUserListElement *)m_BlockedUsersList.GetTail();
	CBlockedUserListElement *pNextElement  = NULL;

	// Store the head position
	POSITION pos = m_BlockedUsersList.GetHeadPosition();

	while(1)
	{
		// If this user name matches a list owner name
		if(pHeadElement->m_strUser == ownerName)
		{
			// Create a new blocked user entry (message packet) and return;
			CreateBlockedUserMessagePacket(userName, pHeadElement);
			return;
		}	

		// If we ve reached end of list
		if(pHeadElement == pTailElement)
			break;

		// Else go ahead 1 position
		pHeadElement = (CBlockedUserListElement *)m_BlockedUsersList.GetNext(pos);
	}

	// If we reach here, it means there's no blocked list for this owner. In that case
	// we create a new one
	CreateBlockedUserElement(ownerName, userName);
}



/**************************************************************************/
// Method Name	: CreateBlockedUserElement
// Description	: Method for creating an element in a blocked user's list
// Inputs		: User name who is the owner of blocked users list
//				  User whose name is to be added in blocked users list
// Output		: Creates a blocked users list element.
//				  Adds the input user name as one of the blocked users under that list
// Returns		: void
/**************************************************************************/
CBlockedUserListElement *CMessengerServerDoc::CreateBlockedUserElement(CString& ownerName, CString& userName)
{
	// Create an element
	CBlockedUserListElement *pElement = new CBlockedUserListElement;
	pElement->m_strUser = ownerName;

	// Add this element to the blocked users list
	m_BlockedUsersList.AddTail(pElement);
	
	// Add new blocked user's name to the element
	CreateBlockedUserMessagePacket(userName, pElement);

	return pElement;
}



/**************************************************************************/
// Method Name	: CreateBlockedUserMessagePacket
// Description	: Method for creating a message packet under an element in 
//					a blocked user's list
// Inputs		: User name who is the new member of blocked users list
//				  Element under whom the new name is to be added as blocked
// Output		: Creates a message packet
//				  Adds the input user name as from field inside the msg packet
// Returns		: void
/**************************************************************************/
void CMessengerServerDoc::CreateBlockedUserMessagePacket(CString &userName, CBlockedUserListElement *element)
{
	// Add new blocked user's name to the element
	CMessagePacket *pMsg = new CMessagePacket;
	pMsg->m_strFrom = userName;
	element->m_BlockedUsers.AddTail(pMsg);
}



/**************************************************************************/
// Method Name	: CreateBlockedUsersList
// Description	: Method for creating Blocked Users List
// Inputs		: None
// Output		: Reads all the blocked users names from database
//				  Creates blocked user message packets and adds them to the blocked
//					users element
// Returns		: void
/**************************************************************************/
void CMessengerServerDoc::CreateBlockedUsersList()
{			
	CBlockedUserListElement *currentElement = NULL;	// Element owned by this user
	CBlockedUserRecord m_BlockedUserRec;

	TRY
	{
		if(m_BlockedUserRec.IsOpen())
		{
			m_BlockedUserRec.Close();
		}

		// Query to get all the friends of this user
		CString strQuery = "Select UserName, BlockedUserName from BlockedUser order by UserName";

		m_BlockedUserRec.Open(AFX_DB_USE_DEFAULT_TYPE, strQuery);

		// Check whether it's an empty recordset
		if(m_BlockedUserRec.IsBOF())
			return;

		// Store the fisrt user's name
		CString user = "";

		// Traverse thru the record set
		while(!m_BlockedUserRec.IsEOF())
		{
			// Create the blocked users list element if next user
			if(user != m_BlockedUserRec.m_UserName)
			{
				currentElement = CreateBlockedUserElement(m_BlockedUserRec.m_UserName, m_BlockedUserRec.m_BlockedUserName);
				user = m_BlockedUserRec.m_UserName;
			}
			else	// If element is already present
				// Create a message packet with this user's name and add to the list
				CreateBlockedUserMessagePacket(m_BlockedUserRec.m_BlockedUserName, currentElement);

			m_BlockedUserRec.MoveNext();
		}

		// Close the record set
		m_BlockedUserRec.Close();
	}
	CATCH(CDBException, edb)
	{
		char err[100];
		edb->GetErrorMessage(err, 100);
		::AfxMessageBox("DB Exception = ");
		::AfxMessageBox(err);
	}
	CATCH(CMemoryException, emem)
	{
		char err[100];
		emem->GetErrorMessage(err, 100);
		::AfxMessageBox("Memory Exception = ");
		::AfxMessageBox(err);
	}
	END_CATCH
}



/**************************************************************************/
// Method Name	: RemoveTempBlockedUsers
// Description	: Method for removing some elements from Blocked Users List
// Inputs		: None
// Output		: Finds out the branch in the blocked users list where
//					the message sender is an owner
//				  If found, removes all the temp blocked users under that branch
//				  If the branch is empty after this, removes the branch, too
// Returns		: void
/**************************************************************************/
void CMessengerServerDoc::RemoveTempBlockedUsers(CMessagePacket &msg)
{
	// If list is empty, no need to go further
	if(m_BlockedUsersList.IsEmpty())
		return;

	// Get addresses of head and tail members of the list
	CBlockedUserListElement *pHeadElement = (CBlockedUserListElement *)m_BlockedUsersList.GetHead();
	CBlockedUserListElement *pTailElement = (CBlockedUserListElement *)m_BlockedUsersList.GetTail();
	CBlockedUserListElement *pNextElement = NULL;

	// Store the head position
	POSITION pos = m_BlockedUsersList.GetHeadPosition();
	pHeadElement = (CBlockedUserListElement *)m_BlockedUsersList.GetNext(pos);

	while(1)
	{
		if(pHeadElement != pTailElement)
			pNextElement = (CBlockedUserListElement *)m_BlockedUsersList.GetNext(pos);

		// If we have reached the right branch, iterated under that branch
		if(pHeadElement->m_strUser == msg.m_strFrom)
		{
			// If this user hasn't blocked any users, return
			if(pHeadElement->m_BlockedUsers.IsEmpty())
				return;

			// Get addresses of head and tail members of the blocked users list
			CMessagePacket *pHeadMsg = (CMessagePacket *)pHeadElement->m_BlockedUsers.GetHead();
			CMessagePacket *pTailMsg = (CMessagePacket *)pHeadElement->m_BlockedUsers.GetTail();
			CMessagePacket *pNextMsg = NULL;

			// Store the head position
			POSITION pos = pHeadElement->m_BlockedUsers.GetHeadPosition();
			pHeadMsg = (CMessagePacket *)pHeadElement->m_BlockedUsers.GetNext(pos);

			while(1)
			{
				if(pHeadMsg != pTailMsg)
					pNextMsg = (CMessagePacket *)pHeadElement->m_BlockedUsers.GetNext(pos);

				// Check if current user name is a temp blocked user, 
				// if yes, delete him
				// Temp blocked user names have a '*' char appended to them
				if(pHeadMsg->m_strFrom.GetAt(pHeadMsg->m_strFrom.GetLength() - 1) == '*')
				{
					pos = pHeadElement->m_BlockedUsers.Find(pHeadMsg, pos);

					// Remove this element from the list first
					pHeadElement->m_BlockedUsers.RemoveAt(pos);

					// Delete the memory pointed to by pHeadMsg
					delete(pHeadMsg);
				}
				
				// If we ve reached end of list
				if(pHeadMsg == pTailMsg)
					break;

				// Else go ahead 1 position
				pHeadMsg = pNextMsg;
			}
		}

		// Now since we have deleted the necessary blocked users 
		// in this list, we check whether we need to keep the head element
		if(pHeadElement->m_BlockedUsers.IsEmpty())
		{
			// Get position
			POSITION pos = m_BlockedUsersList.Find(pHeadElement);

			// Remove head element from list
			m_BlockedUsersList.RemoveAt(pos);

			// Delete the memory pointed to by pHeadElement
			delete(pHeadElement);
		}
		
		// If we ve reached end of list
		if(pHeadElement == pTailElement)
			break;

		// Else go ahead 1 position
		pHeadElement = pNextElement;
	}
	
}
