//////////////////////////////////////////////////////////////////////
// MessengerServerDoc.h : interface of the CMessengerServerDoc class
// This class contains data which is common to all the clients and 
// methods to act on that data
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_MESSENGERSERVERDOC_H__89024732_0C28_11D6_80F4_0001031200E7__INCLUDED_)
#define AFX_MESSENGERSERVERDOC_H__89024732_0C28_11D6_80F4_0001031200E7__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <afxmt.h>
#include <fstream.h>

#include "ClientThread.h"
#include "MessengerServerSocket.h"
#include "MessengerClientSocket.h"
#include "MessagePacket.h"
#include "UserExported.h"
#include "MessengerClientSocket.h"
#include "UserRecord.h"
#include "MessengerServerView.h"
#include "MainFrm.h"
#include "BlockedUserListElement.h"

class CMessengerServerDoc : public CDocument
{
protected: // create from serialization only
	CMessengerServerDoc();
	DECLARE_DYNCREATE(CMessengerServerDoc)

// Attributes
public:
	// Socket Specifications will be stored here
	int						m_nPortNum;

	CMessengerServerSocket	*m_pServerSocket;	// The only server socket
	CPtrList				m_ClientsList;		// Clients connected and logged in
	CPtrList				m_SocketsList;		// Clients connected but not logged in
	CMessengerClientSocket *m_pClientSocket;	// One for each client
	CPtrList				m_OfflineMessagesList;	// Common for all clients
	CPtrList				m_BlockedUsersList;		// Blocked users list, each element is a list

	// These handles to mutexes are used for synchronized access to diff vars in 
	// doc class which will be modified by diff client threads
	HANDLE					m_hSocketsListMutex;
	HANDLE					m_hClientsListMutex;
	HANDLE					m_hOfflineMessagesListMutex;
	HANDLE					m_hBlockedUsersListMutex;

// Operations
public:

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CMessengerServerDoc)
	public:
	virtual BOOL OnNewDocument();
	virtual void Serialize(CArchive& ar);
	protected:
	virtual BOOL SaveModified();
	//}}AFX_VIRTUAL

// Implementation
public:
	int processAccept();
	virtual ~CMessengerServerDoc();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

public:
	void RemoveTempBlockedUsers(CMessagePacket& msg);
	void AddUserToThisUsersBlockedList(CString& ownerName, CString& userName);
	void SendServerClosingNotificationToListMembers(CPtrList& list);
	bool SendMessagePacket(CMessagePacket& msgPacket, CMessengerClientSocket *clientSocket);

	void EmptyAllLists();
	void EmptyClientsList();
	void EmptyOfflineMessagesList();
	void EmptySocketsList();
	void EmptyBlockedUsersList();
	void SendServerClosingNotificationToAllClients();
	bool GetConfigData();
	CMessengerServerView *GetViewPtr();
	void CreateBlockedUserMessagePacket(CString& userName, CBlockedUserListElement *element);
	CBlockedUserListElement *CreateBlockedUserElement(CString& ownerName, CString& userName);
	void CreateBlockedUsersList();

// Generated message map functions
protected:
	void EmptyBlockedUsersListElement(CBlockedUserListElement *element);
	//{{AFX_MSG(CMessengerServerDoc)
		// NOTE - the ClassWizard will add and remove member functions here.
		//    DO NOT EDIT what you see in these blocks of generated code !
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_MESSENGERSERVERDOC_H__89024732_0C28_11D6_80F4_0001031200E7__INCLUDED_)
