// MessengerServerView.cpp : implementation of the CMessengerServerView class
//

#include "stdafx.h"
#include "MessengerServer.h"

#include "MessengerServerDoc.h"
#include "MessengerServerView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMessengerServerView

IMPLEMENT_DYNCREATE(CMessengerServerView, CFormView)

BEGIN_MESSAGE_MAP(CMessengerServerView, CFormView)
	//{{AFX_MSG_MAP(CMessengerServerView)
	ON_BN_CLICKED(IDC_DISCONNECT_USER, OnDisconnectUser)
	ON_WM_SIZE()
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CFormView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CFormView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CFormView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMessengerServerView construction/destruction

CMessengerServerView::CMessengerServerView()
	: CFormView(CMessengerServerView::IDD)
{
	//{{AFX_DATA_INIT(CMessengerServerView)
	m_ServerMsgs = _T("");
	m_nNumOfOnlineUsers = 0;
	m_nNumOfOfflineUsers = 0;
	m_nNumOfConnections = 0;
	//}}AFX_DATA_INIT
	// TODO: add construction code here
	m_bFirstTime = true;

}

CMessengerServerView::~CMessengerServerView()
{
}

void CMessengerServerView::DoDataExchange(CDataExchange* pDX)
{
	CFormView::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CMessengerServerView)
	DDX_Control(pDX, IDC_SERVER_MSGS, m_ctrlServerMsgs);
	DDX_Control(pDX, IDC_NUM_ONLINE_USERS, m_ctrlNumOfOnlineUsers);
	DDX_Control(pDX, IDC_NUM_OFFLINE_USERS, m_ctrlNumOfOfflineUsers);
	DDX_Control(pDX, IDC_NUM_CONNECTIONS, m_ctrlNumOfConnections);
	DDX_Control(pDX, IDC_DISCONNECT_USER, m_btnDisconnectUsers);
	DDX_Control(pDX, IDC_ONLINE_USERS, m_lstOnlineUsers);
	DDX_Control(pDX, IDC_OFFLINE_USERS, m_lstOffLineUsers);
	DDX_Text(pDX, IDC_SERVER_MSGS, m_ServerMsgs);
	DDX_Text(pDX, IDC_NUM_ONLINE_USERS, m_nNumOfOnlineUsers);
	DDX_Text(pDX, IDC_NUM_OFFLINE_USERS, m_nNumOfOfflineUsers);
	DDX_Text(pDX, IDC_NUM_CONNECTIONS, m_nNumOfConnections);
	//}}AFX_DATA_MAP
}

BOOL CMessengerServerView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CFormView::PreCreateWindow(cs);
}

/**************************************************************************/
// Method Name	: OnInitialUpdate()
// Description	: Called when initializing view
// Inputs		: error code (system defined)
// Output		: Calls base class method
//				  Gets the list of all users from database
// Returns		: void
/**************************************************************************/
void CMessengerServerView::OnInitialUpdate()
{
	CFormView::OnInitialUpdate();
	GetParentFrame()->RecalcLayout();
	ResizeParentToFit();

	// Get all the user names into the offline users list
	GetListOfAllUsers();
}

/////////////////////////////////////////////////////////////////////////////
// CMessengerServerView printing

BOOL CMessengerServerView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CMessengerServerView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CMessengerServerView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

void CMessengerServerView::OnPrint(CDC* pDC, CPrintInfo* /*pInfo*/)
{
	// TODO: add customized printing code here
}

/////////////////////////////////////////////////////////////////////////////
// CMessengerServerView diagnostics

#ifdef _DEBUG
void CMessengerServerView::AssertValid() const
{
	CFormView::AssertValid();
}

void CMessengerServerView::Dump(CDumpContext& dc) const
{
	CFormView::Dump(dc);
}

CMessengerServerDoc* CMessengerServerView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CMessengerServerDoc)));
	return (CMessengerServerDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMessengerServerView message handlers

/**************************************************************************/
// Method Name	: GetListOfAllUsers()
// Description	: Used to read the user database and retrive all user names
// Inputs		: None
// Output		: Gets the list of all users from database
//				  Puts the name into offline users list
// Returns		: void
/**************************************************************************/
void CMessengerServerView::GetListOfAllUsers()
{
	CUserRecord m_UserRec;
	CMessengerServerDoc *pDoc = (CMessengerServerDoc *)GetDocument();

	TRY
	{
		// Close rec if already open (just a precaution)
		if(m_UserRec.IsOpen())
		{
			m_UserRec.Close();
		}

		// Query to get all the users
		CString strQuery = "Select * from User";
		m_UserRec.Open(AFX_DB_USE_DEFAULT_TYPE, strQuery);

		// Check whether it's an empty recordset
		if(m_UserRec.IsBOF())
			return;

		// Traverse thru the record set
		while(!m_UserRec.IsEOF())
		{
			// Put this name in the list
			m_lstOffLineUsers.InsertItem(m_nNumOfOfflineUsers, m_UserRec.m_UserName);
			m_nNumOfOfflineUsers ++;

			m_UserRec.MoveNext();
		}

		// Close the record set
		m_UserRec.Close();
	}
	CATCH(CDBException, edb)
	{
		char err[100];
		edb->GetErrorMessage(err, 100);
		::AfxMessageBox("DB Exception = ");
		::AfxMessageBox(err);
	}
	CATCH(CMemoryException, emem)
	{
		char err[100];
		emem->GetErrorMessage(err, 100);
		::AfxMessageBox("Memory Exception = ");
		::AfxMessageBox(err);
	}
	END_CATCH
}



/**************************************************************************/
// Method Name	: OnDisconnectUser()
// Description	: Called when admin clicks Disconnect Selected Users button
// Inputs		: None
// Output		: Sends a disconnect message to the selected client
// Returns		: void
/**************************************************************************/
void CMessengerServerView::OnDisconnectUser() 
{
	CMessengerServerDoc *pDoc = (CMessengerServerDoc *)GetDocument();

	// If nothing is selected, don't go ahead
	int selCount = m_lstOnlineUsers.GetSelectedCount();
	if(!selCount)
		return;

	// Temp Message packet
	CMessagePacket msg;
	msg.m_strReasonFlag = "ZX";

	// If at least one item is selected, get the first selected item
	POSITION pos = m_lstOnlineUsers.GetFirstSelectedItemPosition();
	int selItemIndex = m_lstOnlineUsers.GetNextItem(-1, LVNI_SELECTED);

	// Send this msg packet to everybody in the list
	for(int i = 0; i < selCount; i++)
	{
		// Get text of selected item. This is the user id
		msg.m_strTo = m_lstOnlineUsers.GetItemText(selItemIndex, 0);

		// Get socket for this user
		CMessengerClientSocket *sock = CClientThread::FindOutTheClient(msg, pDoc);

		// If found, send the message to that socket
		if(sock)
			pDoc->SendMessagePacket(msg, sock);

		// Get the next selected item position. If no item is selected
		// after this, break
		m_lstOnlineUsers.GetNextSelectedItem(pos);
		if(!pos)
			break;

		// Get the text of selected item. This will be friend's id
		selItemIndex = m_lstOnlineUsers.GetNextItem(selItemIndex, LVNI_SELECTED);
	}
}



/**************************************************************************/
// Method Name	: AdjustFormControls()
// Description	: Method to maintain the aspect ratio and relative distances 
//					of various controls on the form
// Inputs		: None
// Output		: Adjust the form elements according to their design and 
//					relative sizes on the form.
//				  Sets the flag indicating this form has been resized at
//					least once.
// Returns		: void
/**************************************************************************/
void CMessengerServerView::AdjustFormControls()
{
	// These rects will contain the coordinates of speak window
	// and that of various controls
	CRect rectWin, rectLogWindow, rectOnlineList, rectOfflineList, rectDisconnectBtn;
	CRect rectOnline, rectOffline, rectConnections;

	// Get main window's rects
	GetWindowRect(&rectWin);
	int winWd = rectWin.Width();
	int winHt = rectWin.Height();

	/****************************************************************************/
	// Position the Message log edit box. It's width is more than 1/4th of form size
	/****************************************************************************/
	rectLogWindow.left = rectWin.left + BUFFER_DISTANCE;
	rectLogWindow.right = rectLogWindow.left + (winWd * 2/ 7);
	rectLogWindow.top = rectWin.top + BUFFER_DISTANCE;
	rectLogWindow.bottom = rectLogWindow.top + winHt - (2 * BUFFER_DISTANCE);
	ScreenToClient(&rectLogWindow);

	int logWd = rectLogWindow.Width();
	int logHt = rectLogWindow.Height();
	m_ctrlServerMsgs.MoveWindow(rectLogWindow.left, rectLogWindow.top, logWd, logHt);


	/****************************************************************************/
	// Position the Disconnect User(s) button
	/****************************************************************************/
	/*
	rectDisconnectBtn.bottom = rectWin.bottom - BUFFER_DISTANCE;
	rectDisconnectBtn.top = rectDisconnectBtn.bottom - BTN_WIDTH;
	rectDisconnectBtn.left = rectWin.left + (2 * BUFFER_DISTANCE) + logWd;
	rectDisconnectBtn.right = rectDisconnectBtn.left + BTN_WIDTH;

	m_btnDisconnectUsers.MoveWindow(rectDisconnectBtn.left, rectDisconnectBtn.top, BTN_WIDTH, BTN_HEIGHT);
*/
	// Set this var to false, because window is displayed once
	m_bFirstTime = false;
}



/**************************************************************************/
// Method Name	: OnSize
// Description	: Called when form is resized
// Inputs		: System defined - Type of resize, new width and height
// Output		: If this is the first time form is being resized,
//					adjust the form controls. If this check is not done,
//					program crashes!!
// Returns		: void
/**************************************************************************/
void CMessengerServerView::OnSize(UINT nType, int cx, int cy) 
{
	CFormView::OnSize(nType, cx, cy);

	// Do this only if the window is resized after display
	/*
	if(!m_bFirstTime)
		// Resize controls appropriately and place them on proper positions
		AdjustFormControls();
	else
		m_bFirstTime = false;
		*/
}
